// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

#ifndef __CONCERT_ilocollectioniH
#define __CONCERT_ilocollectioniH

#if !defined(__CONCERT_ilocollectionH)
#include <ilconcert/ilocollection.h>
#endif

#if !defined(__CONCERT_ilosysH)
#include <ilconcert/ilosys.h>
#endif
#if !defined(__CONCERT_ilohashH)
#include <ilconcert/ilohash.h>
#endif
#if !defined(__CONCERT_iloanyH)
#include <ilconcert/iloany.h>
#endif
#if !defined(__CONCERT_ilolinkedlistH)
#include <ilconcert/ilolinkedlist.h>
#endif

class IloInt2IndexHashTable;
class IloNum2IndexHashTable;
class IloAny2IndexHashTable;

class IloMapItem;

class IloDataCollectionI;
class IloContinuousDataI;
class IloNumRangeI;
class IloDiscreteDataCollectionI;
class IloIntCollectionI;
class IloIntAbstractSetI;
class IloIntSetI;
class IloIntRangeI;
class IloNumCollectionI;
class IloNumAbstractSetI;
class IloNumSetI;
class IloAnyCollectionI;
class IloAnyAbstractSetI;
class IloAnySetI;

class IloDataCollection;
class IloContinuousData;
class IloNumRange;
class IloDiscreteDataCollection;
class IloIntCollection;
class IloIntAbstractSet;
class IloIntSet;
class IloIntRange;
class IloNumCollection;
class IloNumAbstractSet;
class IloNumSet;
class IloAnyCollection;
class IloAnyAbstractSet;
class IloAnySet;

class IloDataIterator;

class IloExtendedComprehensionI;

class IloNumVarDomainI;
class IloIntVarDomainI;


class IloDataException : public IloException {
public:
	IloDataException(const char * m) : IloException(m) { }
};

////////////////////////////////////////////////////////////////////////////
//
// IloData
//
////////////////////////////////////////////////////////////////////////////

class IloIntArray;
class IloNumArray;
class IloDataIterator;


class IloDataCollectionI : public IloRttiEnvObjectI {
	ILORTTIDECL
  ILOEXTENSIONI
private:
	IloBool _lock;

public:

	virtual void setLockable(IloBool flag){
		_lock = flag;
	}

	IloBool isLocked() {
		return _lock;
	}

	static void sort(IloIntArray& array, IloInt min, IloInt max);

	static void sort(IloIntArray& array);

	static void sort(IloNumArray& array, IloInt min, IloInt max);

	static void sort(IloNumArray& array);

public:

	IloDataCollectionI(IloEnv env);

	virtual ~IloDataCollectionI();
	virtual IloDataCollectionI* copy() const = 0;

		virtual IloBool includes(IloDataCollectionI* dom) const = 0;

		virtual IloDataType getDataType() const = 0;

		virtual IloBool isIntRange() const;
		IloIntRange asIntRange() const;

		virtual IloBool isNumRange() const;
		IloNumRange asNumRange() const;

		virtual IloBool isIntSet() const;
		IloIntSet asIntSet() const;

		virtual IloBool isNumSet() const;
		IloNumSet asNumSet() const;

		virtual IloBool isSymbolSet() const;
		IloSymbolSet asSymbolSet() const;

		virtual IloBool isAnySet() const;
		IloAnySet asAnySet() const;

		virtual IloBool isIntDataColumn() const;
		IloIntDataColumn asIntDataColumn() const;

		virtual IloBool isNumDataColumn() const;
		IloNumDataColumn asNumDataColumn() const;

		virtual IloBool isAnyDataColumn() const;
		IloAnyDataColumn asAnyDataColumn() const;

		virtual IloBool isTupleCollection() const;
		IloTupleCollection asTupleCollection() const;

		virtual IloBool isTupleSet() const;
		IloTupleSet asTupleSet() const;

		virtual IloBool isTupleRefColumn() const;
		IloIntDataColumn asTupleRefColumn() const;

		virtual IloBool isIntCollectionColumn() const;

		virtual IloBool isNumCollectionColumn() const;

		virtual IloBool isAnyCollectionColumn() const;

		virtual IloBool isIntCollection() const{ return IloFalse; }
		virtual IloBool isNumCollection() const{ return IloFalse; }
		virtual IloBool isAnyCollection() const{ return IloFalse; }

		virtual void displayType(ILOSTD(ostream)& os) const=0;

		virtual void display(ILOSTD(ostream)& os) const=0;

		virtual void displayContent(ILOSTD(ostream)& os) const;
};

//-------------------------------------------------------------------------
#if defined(ILO_USE_SPARSE_ARRAYS)
inline void IloDataCollectionI::sort(IloIntArray& array){
	sort(array, 0, array.getSize()-1);
}
inline void IloDataCollectionI::sort(IloNumArray& array){
	sort(array, 0, array.getSize()-1);
}
#endif

inline ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out,IloDataCollectionI* dom) {
	if (dom){
		dom->display(out);
	}
	return (out);
}

class IloDiscreteDataCollectionI : public IloDataCollectionI {
	ILORTTIDECL

public:

	virtual IloBool hasReference() const { return IloFalse; }
	virtual IloInt getSize() const = 0;

	virtual IloMapItem getMapItem(IloInt idx) const = 0;
	IloDiscreteDataCollectionI(IloEnv env): IloDataCollectionI(env) {}

	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;
	virtual IloDataIterator* iterator() const;

	virtual IloInt getMapItemIndex(IloMapItem value) const;

	virtual IloInt getIndex(IloInt value) const;
	virtual IloInt getIndex(IloNum value) const;
	virtual IloInt getIndex(IloAny value) const;
	virtual IloInt getIndex(IloSymbol value) const;

	virtual IloInt getIntValue(IloInt idx) const;
	virtual IloNum getNumValue(IloInt idx) const;
	virtual IloAny getAnyValue(IloInt idx) const;
	IloSymbol getSymbolValue(IloInt idx) const;
	const char* getStringValue(IloInt idx) const;


	virtual IloBool supportDuplicates() const;
	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const = 0;
};


class IloContinuousDataI : public IloDataCollectionI {
	ILORTTIDECL
public:
	IloContinuousDataI(IloEnv env): IloDataCollectionI(env) {}
};


class IloNumRangeI : public IloContinuousDataI {
	ILORTTIDECL
protected:
	IloNum _min;
	IloNum _max;
public:
	IloVarDomainI* getSharedDomain() const;
	IloNumRangeI(IloEnv env, IloNum min, IloNum max);

	IloNumRangeI(IloNumRangeI* obj);

	virtual ~IloNumRangeI();

	virtual IloDataCollectionI* copy() const;

	IloBool includes(IloDataCollectionI* dom) const;

	virtual IloDataType getDataType() const;

	virtual IloBool isNumRange() const;

	virtual void displayType(ILOSTD(ostream)& os) const;

	void setUB(IloNum val){
		_max = val;
	}

	void setLB(IloNum val){
		_min = val;
	}

	void display(ILOSTD(ostream)& os) const{
		os << "[";
		os << _min << " ," << _max;
		os << "]";
	}

	IloNum getLB() const {
		return _min;
	}
	IloNum getUB() const {
		return _max;
	}
};


class IloIntCollectionI : public IloDiscreteDataCollectionI {
	ILORTTIDECL
public:
	IloIntCollectionI(IloEnv env): IloDiscreteDataCollectionI(env) {}

    virtual IloInt getIndex(IloSymbol s) const;
	virtual IloInt getIndex(IloInt value) const;
	virtual IloInt getIndex(IloNum value) const;
	virtual IloInt getIndex(IloAny value) const;

	virtual IloInt getIntValue(IloInt idx) const;
	virtual IloNum getNumValue(IloInt idx) const;
	virtual IloAny getAnyValue(IloInt idx) const;

	virtual  const IloInt& operator[] (IloInt i) const;

	virtual IloIntArray getArray() const = 0;

	virtual IloBool contains(IloInt e) const=0;
	virtual IloInt getValue(IloInt index) const=0;
	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const {
		out << getValue(index);
	}
	virtual IloBool isIntCollection() const { return IloTrue; }
};

class IloIntAbstractSetI : public IloIntCollectionI {
	ILORTTIDECL
public:
	IloIntAbstractSetI(IloEnv env): IloIntCollectionI(env) {}
	virtual IloInt getNext(IloInt value, IloInt n=1) const = 0;
	virtual IloInt getPrevious(IloInt value, IloInt n=1) const = 0;

	virtual IloInt getNextC(IloInt value, IloInt n=1) const = 0;
	virtual IloInt getPreviousC(IloInt value, IloInt n=1) const = 0;
	virtual IloInt getFirst() const = 0;
	virtual IloInt getLast() const = 0;
	virtual IloBool supportDuplicates() const;

	virtual IloInt getLB() const;
	virtual IloInt getUB() const;
};


class IloIntAbstractDataColumnI : public IloIntCollectionI {
	ILORTTIDECL
public:
	IloIntAbstractDataColumnI(IloEnv env): IloIntCollectionI(env) {}

	virtual IloIntArray makeIndexArray(IloInt value) const = 0;
};


class IloNumCollectionI : public IloDiscreteDataCollectionI {
	ILORTTIDECL
public:
	IloNumCollectionI(IloEnv env): IloDiscreteDataCollectionI(env) {}

	virtual IloNumArray getArray() const = 0;

        virtual IloInt getIndex(IloSymbol s) const;
	virtual IloInt getIndex(IloInt value) const;
	virtual IloInt getIndex(IloNum value) const;
	virtual IloInt getIndex(IloAny value) const;

	virtual IloInt getIntValue(IloInt idx) const;
	virtual IloNum getNumValue(IloInt idx) const;
	virtual IloAny getAnyValue(IloInt idx) const;

	virtual  const IloNum& operator[] (IloInt i) const;

	virtual IloBool contains(IloNum e) const=0;
	virtual IloNum getValue(IloInt index) const=0;
	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const {
		out << getValue(index);
	}
	virtual IloBool isNumCollection() const { return IloTrue; }
};


class IloNumAbstractSetI : public IloNumCollectionI {
	ILORTTIDECL
public:
	IloNumAbstractSetI(IloEnv env): IloNumCollectionI(env) {}

        IloInt getIndex(IloSymbol s) const;
        IloInt getIndex(IloAny s) const;
        IloInt getIndex(IloNum s) const;

	IloInt getIndex(IloInt value) const;

	virtual IloNum getNext(IloNum value, IloInt n=1) const = 0;
	virtual IloNum getPrevious(IloNum value, IloInt n=1) const = 0;
	virtual IloNum getNextC(IloNum value, IloInt n=1) const = 0;
	virtual IloNum getPreviousC(IloNum value, IloInt n=1) const = 0;

	virtual IloNum getFirst() const = 0;
	virtual IloNum getLast() const = 0;

	virtual IloBool supportDuplicates() const;

	virtual IloNum getLB() const;
	virtual IloNum getUB() const;
};

class IloNumAbstractDataColumnI : public IloNumCollectionI {
	ILORTTIDECL
public:
	IloNumAbstractDataColumnI(IloEnv env): IloNumCollectionI(env) {}

	virtual IloIntArray makeIndexArray(IloNum value) const = 0;
};


class IloAnyCollectionI : public IloDiscreteDataCollectionI {
	ILORTTIDECL
public:
	IloAnyCollectionI(IloEnv env): IloDiscreteDataCollectionI(env) {}
	virtual IloAnyArray getArray() const = 0;

        virtual IloInt getIndex(IloSymbol s) const;
	virtual IloInt getIndex(IloInt value) const;
	virtual IloInt getIndex(IloNum value) const;
	virtual IloInt getIndex(IloAny value) const;
	virtual IloInt getIntValue(IloInt idx) const;
	virtual IloNum getNumValue(IloInt idx) const;
	virtual IloAny getAnyValue(IloInt idx) const;

	virtual  const IloAny& operator[] (IloInt i) const;
	virtual IloBool contains(IloAny e) const=0;
	virtual IloAny getValue(IloInt index) const=0;
	virtual IloBool isAnyCollection() const { return IloTrue; }
};

class IloAnyAbstractSetI : public IloAnyCollectionI {
	ILORTTIDECL
		IloDataType _type;
public:
	const IloDataType getType() const { return _type;}
	IloAnyAbstractSetI(IloEnv env): IloAnyCollectionI(env), _type(AnySet) {}
	void setType(IloDataType type){ _type = type; }

	virtual IloAny getNext(IloAny value, IloInt n=1) const = 0;
	virtual IloAny getPrevious(IloAny value, IloInt n=1) const = 0;
	virtual IloAny getNextC(IloAny value, IloInt n=1) const = 0;
	virtual IloAny getPreviousC(IloAny value, IloInt n=1) const = 0;

	virtual IloAny getFirst() const = 0;
	virtual IloAny getLast() const = 0;

	virtual IloBool supportDuplicates() const;
};



class IloAnyAbstractDataColumnI : public IloAnyCollectionI {
	ILORTTIDECL
public:
	IloAnyAbstractDataColumnI(IloEnv env): IloAnyCollectionI(env) {}

	virtual IloIntArray makeIndexArray(IloAny value) const = 0;
};



class IloIntRangeI : public IloIntAbstractSetI {
	ILORTTIDECL
protected:
	IloInt _min;
	IloInt _max;
public:
	IloVarDomainI* getSharedDomain() const;
	virtual IloDataType getDataType() const;

	virtual IloDataCollectionI* copy() const;

	virtual IloInt getNext(IloInt value, IloInt n=1) const;
	virtual IloInt getPrevious(IloInt value, IloInt n=1) const;

	virtual IloInt getNextC(IloInt value, IloInt n=1) const;
	virtual IloInt getPreviousC(IloInt value, IloInt n=1) const;

	virtual IloInt getFirst() const;
	virtual IloInt getLast() const;

	virtual IloInt getMapItemIndex(IloMapItem value) const;


	IloInt getIndex(IloInt value) const;

	IloInt getIndex(IloAny value) const;
	IloInt getIndex(IloNum value) const;
	IloInt getIndex(IloSymbol value) const;

	IloInt getIntValue(IloInt idx) const;

	IloIntArray getArray() const;

	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;

	virtual IloDataIterator* iterator() const;

	IloIntRangeI(IloEnv env);

	IloIntRangeI(IloEnv env, IloInt min, IloInt max);

	IloIntRangeI(IloIntRangeI* obj);

	virtual ~IloIntRangeI();


	virtual IloBool includes(IloDataCollectionI* dom) const;

	virtual IloBool isIntRange() const;

	virtual void displayType(ILOSTD(ostream)& os) const;

	virtual void display(ILOSTD(ostream)& os) const;

	virtual IloInt getSize() const;
	virtual IloMapItem getMapItem(IloInt idx) const;

	virtual IloInt getLB() const {
		return _min;
	}

	virtual IloInt getUB() const {
		return _max;
	}

	virtual void setLB(IloInt min);

	virtual void setUB(IloInt max);

	virtual IloBool contains(IloInt e) const;
	virtual IloInt getValue(IloInt index) const;
};


class IloIntSetI : public IloIntAbstractSetI {
	ILORTTIDECL
private:
	IloBool _hasIndex;
	IloIntArray _array;
	IloInt2IndexHashTable* _hashForGetIndex;
	void createHashTable();
public:
	IloInt2IndexHashTable* getOrMakeHashTable();
	virtual void removeIndexes();
	void updateIndexes();
  void clearAllCaches();
protected:
	void removeFromArray(IloInt elt);
	void addWithoutCheck(IloInt elt);

public:
	void build(IloExtendedComprehensionI* comp, IloModelEvaluatorI* eval=0);
	virtual IloDataCollectionI* copy() const;

	const IloInt& operator[] (IloInt i) const;

	virtual IloInt getNext(IloInt value, IloInt n=1) const;
	virtual IloInt getPrevious(IloInt value, IloInt n=1) const;
	virtual IloInt getNextC(IloInt value, IloInt n=1) const;
	virtual IloInt getPreviousC(IloInt value, IloInt n=1) const;

	virtual IloInt getFirst() const;
	virtual IloInt getLast() const;

	virtual IloInt getMapItemIndex(IloMapItem value) const;

	IloInt getIndex(IloInt value) const;

	IloInt getIndex(IloSymbol value) const;
	IloInt getIndex(IloNum value) const;

	IloInt getIndex(IloAny value) const;

	IloInt getIntValue(IloInt idx) const;

	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;

	virtual IloDataIterator* iterator() const;

	IloIntSetI(IloEnvI* env, const IloIntArray array, IloBool withIndex = IloFalse);

	IloIntSetI(IloEnvI* env, IloBool withIndex = IloFalse);

	IloIntSetI(IloEnvI* env,  IloIntSetI* set);

	virtual ~IloIntSetI();

	IloBool hasIndex() const {
		return  _hasIndex;
	}

	IloIntArray getArray() const {return _array;}

	IloInt getSize() const;
	virtual IloMapItem getMapItem(IloInt idx) const;

	void add(IloInt elt);

	void add(IloIntSetI* set);

	void remove(IloInt elt);

	void remove(IloIntSetI* set);

	void setIntersection(IloInt elt);

	void setIntersection(IloIntSetI* set);

	void empty();

	virtual IloBool contains(IloInt elt) const;

	virtual IloInt getValue(IloInt index) const;

	IloBool contains(IloIntSetI* set) const;

	IloBool intersects(IloIntSetI* set) const;

	IloNumSetI * toNumSet() const;

	IloAnySetI * toAnySet() const;

	IloIntArray toArray() const { return _array.copy(); }

	IloDataType getDataType() const;

	IloBool includes(IloDataCollectionI* dom) const;

	IloBool isIntSet() const;

	void displayType(ILOSTD(ostream)& os) const;

	void display(ILOSTD(ostream)& os) const;

  static IloIntSetI* MakeDiff(IloIntSetI* set1, IloIntSetI* set2);

};

class IloNumSetI : public IloNumAbstractSetI {
	ILORTTIDECL
private:
	IloBool _hasIndex;
	IloNumArray _array;
	IloNum2IndexHashTable* _hashForGetIndex;
	void createHashTable();
public:
	IloNum2IndexHashTable* getOrMakeHashTable();

	virtual void removeIndexes();
	void updateIndexes();
  void clearAllCaches();
protected:
	void removeFromArray(IloNum elt);
	//virtual void createIndexesFromArray();
	void addWithoutCheck(IloNum elt);

public:
	void build(IloExtendedComprehensionI* comp, IloModelEvaluatorI* eval=0);
	virtual IloDataCollectionI* copy() const;

	virtual IloNum  getNext(IloNum value, IloInt n=1) const;
	virtual IloNum  getPrevious(IloNum value, IloInt n=1) const;

	virtual IloNum  getNextC(IloNum value, IloInt n=1) const;
	virtual IloNum  getPreviousC(IloNum value, IloInt n=1) const;

	virtual IloNum getFirst() const;
	virtual IloNum getLast() const;

	const IloNum& operator[] (IloInt i) const;

	virtual IloInt getMapItemIndex(IloMapItem value) const;

	IloInt getIndex(IloNum value) const;

	IloInt getIndex(IloInt value) const;
	IloInt getIndex(IloAny value) const;
	IloInt getIndex(IloSymbol value) const;

	IloNum getNumValue(IloInt idx) const;

	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;

	virtual IloDataIterator* iterator() const;

	IloNumSetI(IloEnvI* env, const IloNumArray array, IloBool withIndex = IloFalse);

	IloNumSetI(IloEnvI* env, IloBool withIndex = IloFalse);

	IloNumSetI(IloEnvI* env,  IloNumSetI* set);

	virtual ~IloNumSetI();

	IloBool hasIndex() const {
		return  _hasIndex;
	}
	IloNumArray getArray() const {return _array;}

	IloInt getSize() const;
	virtual IloMapItem getMapItem(IloInt idx) const;

	void add(IloNum elt);

	void add(IloNumSetI* set);

	void add(IloIntSetI* set);

	void remove(IloNum elt);

	void remove(IloNumSetI* set);

	void setIntersection(IloNum elt);

	void setIntersection(IloNumSetI* set);

	void empty();

	virtual IloBool contains(IloNum elt) const;

	virtual IloNum getValue(IloInt index) const;

	IloBool contains(IloNumSetI* set) const;

	IloBool intersects(IloNumSetI* set) const;

	IloIntSetI * toIntSet() const;

	IloNumArray toArray() const { return _array.copy(); }

	IloDataType getDataType() const;

	IloBool includes(IloDataCollectionI* dom) const;

	IloBool isNumSet() const;

	void displayType(ILOSTD(ostream)& os) const;

	void display(ILOSTD(ostream)& os) const;

  static IloNumSetI* MakeDiff(IloNumSetI* set1, IloNumSetI* set2);
};


class IloAnySetI : public IloAnyAbstractSetI {
	ILORTTIDECL
private:
	IloBool _hasIndex;
	IloAnyArray _array;
	IloAny2IndexHashTable* _hashForGetIndex;
	void createHashTable();
public:
	IloAny2IndexHashTable* getOrMakeHashTable();

	virtual void removeIndexes();
	void updateIndexes();
  void clearAllCaches();
protected:
	void removeFromArray(IloAny elt);
public:
	void addWithoutCheck(IloAny elt);
public:
	void build(IloExtendedComprehensionI* comp, IloModelEvaluatorI* eval=0);
	virtual IloDataCollectionI* copy() const;

	virtual IloAny getNext(IloAny value, IloInt n=1) const;
	virtual IloAny getPrevious(IloAny value, IloInt n=1) const;

	virtual IloAny getNextC(IloAny value, IloInt n=1) const;
	virtual IloAny getPreviousC(IloAny value, IloInt n=1) const;

	virtual IloAny getFirst() const;
	virtual IloAny getLast() const;

	const IloAny& operator[] (IloInt i) const;

	IloInt getIndex(IloNum value) const;
	virtual IloInt getMapItemIndex(IloMapItem value) const;

	IloInt getIndex(IloAny value) const;

	IloInt getIndex(IloSymbol value) const;

	IloInt getIndex(IloInt value) const;

	IloAny getAnyValue(IloInt idx) const;

	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;

	virtual IloDataIterator* iterator() const;

	IloAnySetI(IloEnvI* env, const IloAnyArray array, IloBool withIndex = IloFalse);
	IloAnySetI(IloEnvI* env, IloBool withIndex = IloFalse);
	IloAnySetI(IloEnvI* env,  IloAnySetI* set);

	virtual ~IloAnySetI();

	IloBool hasIndex() const {
		return  _hasIndex;
	}

	IloAnyArray getArray() const {return _array;}

	IloInt getSize() const;
	virtual IloMapItem getMapItem(IloInt idx) const;

	void add(IloAny elt);

	void add(IloAnySetI* set);

	void remove(IloAny elt);

	void remove(IloAnySetI* set);

	void setIntersection(IloAny elt);

	void setIntersection(IloAnySetI* set);

	void empty();

	virtual IloBool contains(IloAny elt) const;
	virtual IloAny getValue(IloInt index) const;

	IloBool contains(IloAnySetI* set) const;
	IloBool intersects(IloAnySetI* set) const;
	IloNumSetI * toNumSet() const;
	IloIntSetI * toIntSet() const;
	IloAnyArray toArray() const { return _array.copy(); }

	IloDataType getDataType() const;

	IloBool isAnySet() const;
	IloBool isSymbolSet() const;

	virtual void displayType(ILOSTD(ostream)& os) const;

	virtual void display(ILOSTD(ostream)& os) const;

	IloBool includes(IloDataCollectionI* dom) const;

	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const;

        virtual IloEncoderI* getEncoder() const;

  static IloAnySetI* MakeDiff(IloAnySetI* set1, IloAnySetI* set2);
};


class IloIntArrayAsCollectionI : public IloIntCollectionI {
	ILORTTIDECL
private:
	IloIntArray _array;
public:
	virtual IloDataCollectionI* copy() const;

	IloIntArrayAsCollectionI(const IloIntArray array);
	virtual ~IloIntArrayAsCollectionI();
	virtual IloDataType getDataType() const;
	virtual IloBool includes(IloDataCollectionI* dom) const;
	virtual IloInt getSize() const;
	virtual IloMapItem getMapItem(IloInt idx) const;

	virtual IloIntArray getArray() const;
	virtual IloBool contains(IloInt e) const;


	virtual IloInt getIndex(IloNum value) const;
	virtual IloInt getIndex(IloAny value) const;
	virtual IloInt getIndex(IloSymbol value) const;

	virtual IloInt getValue(IloInt index) const;
	virtual IloInt getIndex(IloInt value) const;
	virtual IloInt getIntValue(IloInt idx) const;
	virtual  const IloInt& operator[] (IloInt i) const;
	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;
	virtual IloDataIterator* iterator() const;
	virtual void display(ILOSTD(ostream)& os) const;
	virtual void displayType(ILOSTD(ostream)& os) const;
	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const {
		out << getValue(index);
	}
};

class IloNumArrayAsCollectionI : public IloNumCollectionI {
	ILORTTIDECL
private:
	IloNumArray _array;
public:
	virtual IloDataCollectionI* copy() const;

	IloNumArrayAsCollectionI(const IloNumArray array);
	virtual ~IloNumArrayAsCollectionI();
	virtual IloDataType getDataType() const;
	virtual IloBool includes(IloDataCollectionI* dom) const;
	virtual IloInt getSize() const;
	virtual IloMapItem getMapItem(IloInt idx) const;

	virtual IloNumArray getArray() const;
	virtual const IloNum& operator[] (IloInt i) const;
	virtual IloBool contains(IloNum e) const;
	virtual IloNum getValue(IloInt index) const;


	virtual IloInt getIndex(IloInt value) const;
	virtual IloInt getIndex(IloSymbol value) const;
	virtual IloInt getIndex(IloAny value) const;

	virtual IloInt getIndex(IloNum value) const;
	virtual IloNum getNumValue(IloInt idx) const;

	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;
	virtual IloDataIterator* iterator() const;
	virtual void display(ILOSTD(ostream)& os) const;
	virtual void displayType(ILOSTD(ostream)& os) const;
	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const {
		out << getValue(index);
	}
};

class IloAnyArrayAsCollectionI : public IloAnyCollectionI {
	ILORTTIDECL
private:
	IloAnyArray _array;
public:
	virtual IloDataCollectionI* copy() const;

	IloAnyArrayAsCollectionI(const IloAnyArray array);
	virtual ~IloAnyArrayAsCollectionI();
	virtual IloDataType getDataType() const;
	virtual IloBool includes(IloDataCollectionI* dom) const;
	virtual IloInt getSize() const;
	virtual IloMapItem getMapItem(IloInt idx) const;

	virtual IloAnyArray getArray() const;
	virtual IloBool contains(IloAny e) const;


	virtual IloAny getValue(IloInt index) const;

	virtual IloInt getIndex(IloInt value) const;
	virtual IloInt getIndex(IloNum value) const;
	virtual IloInt getIndex(IloSymbol value) const;

	virtual IloInt getIndex(IloAny value) const;
	virtual IloAny getAnyValue(IloInt idx) const;

	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;
	virtual IloDataIterator* iterator() const;
	virtual void display(ILOSTD(ostream)& os) const;
	virtual void displayType(ILOSTD(ostream)& os) const;
	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const;
};




class IloDummyIntDataIterator : public IloIntDataIterator {
public:
	IloDummyIntDataIterator(IloGenAlloc* heap);
	virtual ~IloDummyIntDataIterator();
	virtual IloBool next();
};

class IloDummyNumDataIterator : public IloNumDataIterator {
public:
	IloDummyNumDataIterator(IloGenAlloc* heap);
	virtual ~IloDummyNumDataIterator();
	virtual IloBool next();
};

class IloDummyAnyDataIterator : public IloAnyDataIterator {
public:
	IloDummyAnyDataIterator(IloGenAlloc* heap);
	virtual ~IloDummyAnyDataIterator();
	virtual IloBool next();
};

//--------------------------------------------------------------


class IloDataChangeListenerI : public IloRttiEnvObjectI {
  ILORTTIDECL
  friend class IloDataChangeNotifierI;
protected :
  virtual void change() const = 0;
public:
  IloDataChangeListenerI(IloEnvI* env) : IloRttiEnvObjectI(env) {}
  virtual ~IloDataChangeListenerI() {};
};

class IloDataChangeNotifierI : public IloRttiEnvObjectI {
  ILORTTIDECL
protected:
  class Listener {
  public:
    Listener(IloDataChangeListenerI* listener, Listener* next) : _listener(listener), _next(next) {}
    IloDataChangeListenerI* _listener;
    Listener* _next;
  };
  Listener* _listeners;
protected:
  virtual void change() const;
public:
  IloDataChangeNotifierI(IloEnvI*);
  void registerListener(IloDataChangeListenerI*);
  void unregisterListener(IloDataChangeListenerI*);
};


#endif


