// -------------------------------------------------------------- -*- C++ -*-
// File: ilconcert/ilodatasource.h
// --------------------------------------------------------------------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

// -*- C++ -*- header file.
#ifndef __CONCERT_ilodatasourceH
#define __CONCERT_ilodatasourceH

#ifndef __CONCERT_ilodatasourceiH
#include <ilconcert/ilsource/ilodatasourcei.h>
#endif



class IloStringDataSource {
private:
	IloStringDataSourceI* _impl;
public:
	IloStringDataSource(IloEnv env);
	IloStringDataSource(): _impl(0) {}

	IloStringDataSourceI* getImpl() const { return _impl; }
	IloStringDataSource(IloStringDataSourceI* impl): _impl(impl) {}

	void end(){
		if (_impl){
			delete _impl;
			_impl = 0;
		}
	}

	IloIntArray getIntArray(const char* str);

	IloNumArray getNumArray(const char* str);
	IloSymbolArray getSymbolArray(const char* str);

	IloIntSet getIntSet(const char* str);

	IloNumSet getNumSet(const char* str);
	IloSymbolSet getSymbolSet(const char* str);

	IloInt getInt(const char* str);
	IloNum getNum(const char* str);
	IloSymbol getSymbol(const char* str);
};

class IloCsvDataSource {
private:
	IloCsvDataSourceI* _impl;
public:
	IloCsvDataSource(IloEnv env, const char* fileName);
	IloCsvDataSource(): _impl(0) {}

	IloCsvDataSourceI* getImpl() const { return _impl; }
	IloCsvDataSource(IloCsvDataSourceI* impl): _impl(impl) {}


	void end(){
		if (_impl){
			delete _impl;
			_impl = 0;
		}
	}

	IloIntSet getIntSetFromLine(const char* tableName, IloInt line = 0);
	IloIntSet getIntSetFromColumn(const char* tableName, IloInt column = 1);

	IloNumSet getNumSetFromLine(const char* tableName, IloInt line = 0);
	IloNumSet getNumSetFromColumn(const char* tableName, IloInt column = 1);
	IloSymbolSet getSymbolSetFromLine(const char* tableName, IloInt line = 0);
	IloSymbolSet getSymbolSetFromColumn(const char* tableName, IloInt column = 1);


	IloIntArray getIntArrayFromLine(const char* tableName, IloInt line = 0);
	IloIntArray getIntArrayFromColumn(const char* tableName, IloInt column = 1);

	IloNumArray getNumArrayFromLine(const char* tableName, IloInt line = 0);
	IloNumArray getNumArrayFromColumn(const char* tableName, IloInt column = 1);
	IloSymbolArray getSymbolArrayFromLine(const char* tableName, IloInt line = 0);
	IloSymbolArray getSymbolArrayFromColumn(const char* tableName, IloInt column = 1);


	IloInt getIntFromTable(const char* tableName, IloInt x, IloInt y);

	IloNum getNumFromTable(const char* tableName, IloInt x, IloInt y);
	IloSymbol getSymbolFromTable(const char* tableName, IloInt x, IloInt y);

	IloTupleSet getTupleSet(IloTupleSchema schema, const char* tableName);
};


#ifdef ILO_MSVC
#if defined(ILO_WIN32) || defined(ILO_MSVC8)
/////////////////

class IloExcelDataSource {
private:
	IloExcelDataSourceI* _impl;
public:
	IloExcelDataSource(IloEnv env, const char* fileName, IloBool readBolny = IloFalse);
	IloExcelDataSource(): _impl(0) {}

	IloExcelDataSourceI* getImpl() const { return _impl; }
	IloExcelDataSource(IloExcelDataSourceI* impl): _impl(impl) {}


	void end(){
		if (_impl){
			delete _impl;
			_impl = 0;
		}
	}

	IloInt getIntFromTable(const char* range, IloInt x, IloInt y);

	IloNum getNumFromTable(const char* range, IloInt x, IloInt y);
	IloSymbol getSymbolFromTable(const char* range, IloInt x, IloInt y);

	///** <p> Gets an IloTupleSet from a range in the XLS file. </p> **/
	//IloTupleSet getTupleSet(IloTupleSchema schema, const char* range);

    /* @internal Used by YOPL. */
    void readIntMap(IloMapI* result, const char* range);
    /* @internal Used by YOPL. */
    void readNumMap(IloMapI* result, const char* range);
    /* @internal Used by YOPL. */
    void readSymbolMap(IloMapI* result, const char* range);
    /* @internal Used by YOPL. */
    void readTupleMap(IloAbstractTupleMapI* result, const char* range);

    /* @internal Used by YOPL. */
    void readIntSet(IloIntSetI* result, const char* range);

    /* @internal Used by YOPL. */
    void readNumSet(IloNumSetI* result, const char* range);

    /* @internal Used by YOPL. */
    void readSymbolSet(IloAnySetI* result, const char* range);

    /* @internal Used by YOPL. */
    void readTupleSet(IloTupleSetI* result, const char* range);


    /* @internal Used by YOPL. */
    void writeIntSet(IloIntSetI* set, const char* range);

    /* @internal Used by YOPL. */
    void writeNumSet(IloNumSetI* set, const char* range);

    /* @internal Used by YOPL. */
    void writeSymbolSet(IloAnySetI* set, const char* range);

    /* @internal Used by YOPL. */
    void writeTupleSet(IloTupleSetI* set, const char* range);



    /* @internal Used by YOPL. */
    void writeInt(IloInt val, const char* range);

    /* @internal Used by YOPL. */
    void writeNum(IloNum val, const char* range);

    /* @internal Used by YOPL. */
    void writeSymbol(IloSymbolI* val, const char* range);

    /* @internal Used by YOPL. */
    void writeTuple(IloTupleI* tuple, const char* range);



    /* @internal Used by YOPL. */
    void writeIntMap(IloMapI* set, const char* range);
    /* @internal Used by YOPL. */
    void writeNumMap(IloMapI* set, const char* range);
    /* @internal Used by YOPL. */
    void writeSymbolMap(IloMapI* set, const char* range);
    /* @internal Used by YOPL. */
    void writeTupleMap(IloAbstractTupleMapI* set, const char* range);

	void save();
};

#endif //ILO_WIN32
#endif //ILO_MSVC

#endif


