// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_iloforallbaseH
#define __CONCERT_iloforallbaseH

#if !defined(__CONCERT_iloextractableH)
# include <ilconcert/iloextractable.h>
#endif
#if !defined(__CONCERT_iloexpressionH)
# include <ilconcert/iloexpression.h>
#endif
#if !defined(__CONCERT_ilocollectionH)
# include <ilconcert/ilocollection.h>
#endif


class IloIntIndex;
class IloNumIndex;
class IloSymbolIndex;

class IloIntCollectionIndex;
class IloNumCollectionIndex;
class IloSymbolCollectionIndex;

class IloTupleIndex;
class IloTuplePattern;


class IloGeneratorI : public IloExtractableI {
  ILOEXTRDECL
private:
  IloExtractableList _filters;
protected:
  virtual void atRemove(IloExtractableI* user=0, IloAny info = 0);
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  virtual IloBool hasNongroundType() const;
public:
  IloGeneratorI(IloEnvI* env);
  virtual ~IloGeneratorI();
  virtual IloBool isOpen() const;
  virtual IloInt getIndexKey() const=0;
  const IloExtractableList& getFilters() const { return _filters; }
  IloGeneratorI* suchThat(IloConstraintI* cond);
  IloGeneratorI* suchThat(IloBool cond);
  void displayFilters(ILOSTD(ostream)& out) const;
  virtual IloBool generatesDuplicates() const=0;
  virtual IloDataIterator* iterator() const = 0;
  virtual IloDiscreteDataCollectionI* getDiscreteDataCollection() const = 0;
};

//----------------------------------------------
class ILO_EXPORTED IloIndex {
private:
  IloExtractableI* _impl;
public:
  IloIndex():_impl(0) {}
  IloIndex(IloIntIndex x);
  IloIndex(IloNumIndex x);
  IloIndex(IloSymbolIndex x);

  /* <p> . </p> */
  IloIndex(IloIntCollectionIndex x);
  /* <p> . </p> */
  IloIndex(IloNumCollectionIndex x);
  /* <p> . </p> */
  IloIndex(IloSymbolCollectionIndex x);

  IloIndex(IloTupleIndex x);
  IloIndex(IloTuplePattern x);
  ~IloIndex() {}
  IloExtractableI* getImpl() const { return _impl; }
  IloEnv getEnv() const {
    IloAssert(_impl != 0, "Access empty IloIndex handle.");
    return _impl->getEnv();
  }
};



typedef IloArray<IloIndex> IloIndexArrayBase;

class ILO_EXPORTED IloIndexArray : public IloIndexArrayBase {
public:
  typedef IloDefaultArrayI ImplClass;
  IloIndexArray(IloDefaultArrayI* i=0) : IloIndexArrayBase(i) {}
  IloIndexArray(const IloIndexArray& copy) : IloIndexArrayBase(copy) {}
  IloIndexArray(const IloEnv env, IloInt n = 0) : IloIndexArrayBase(env, n) {}
  IloIndexArray(const IloMemoryManager env, IloInt n = 0) : IloIndexArrayBase(env, n) {}
};



/////////////////////////////////////////////////////////////////////////////
// Generators
/////////////////////////////////////////////////////////////////////////////
class ILO_EXPORTED IloGenerator {
private:
  IloGeneratorI* _impl;
public:
  IloGenerator():_impl(0) {}
  IloGenerator(IloGeneratorI* impl):_impl(impl) {}
  ~IloGenerator() {}
  IloGeneratorI* getImpl() const { return _impl; }

  IloGenerator suchThat(IloConstraint ct) const {
    IloAssert(getImpl(), "Using empty IloGenerator handle.");
    IloAssert(ct.getImpl(), "Using empty IloConstraint handle.");
    return (getImpl()->suchThat(ct.getImpl()));
  }

  IloGenerator suchThat(IloBool ct) const {
    IloAssert(getImpl(), "Using empty IloGenerator handle.");
    return (getImpl()->suchThat(ct));
  }

  /////////////////////////////
  // EXCEPTIONS
  /////////////////////////////
  class Exception : public IloException {
  protected:
    const IloExtractableI* _context;
  public:
    Exception(const IloExtractableI* ctx);
    virtual ~Exception();
    IloExtractable getContext() const;
	  void setContext(const IloExtractableI* e);
  };

  class MultipleDefinition : public Exception {
  private:
    const IloInt _key;
  public:
    MultipleDefinition(const IloExtractableI* ctx,
		       const IloInt key);
	  virtual const char* getMessage() const;
    void print(ILOSTD(ostream)& out) const;

  };
  class OutOfScope : public Exception {
  private:
    const IloInt _key;
  public:
    OutOfScope(const IloExtractableI* ctx,
	       const IloInt key);
	  virtual const char* getMessage() const;
    void print(ILOSTD(ostream)& out) const;
  };

  class InvalidFilter : public Exception {
  private:
	  const IloExtractableI* _filterExp;
  public:
	  InvalidFilter(const IloExtractableI* exp);
	  virtual ~InvalidFilter();
	  virtual const char* getMessage() const;
	  virtual void print(ILOSTD(ostream)& out) const;
  };
};

typedef IloArray<IloGenerator> IloGeneratorArrayBase;

class ILO_EXPORTED IloGeneratorArray : public IloGeneratorArrayBase {
public:
  typedef IloDefaultArrayI ImplClass;

  IloGeneratorArray(ImplClass* i=0)
    :IloGeneratorArrayBase(i) {}
  IloGeneratorArray(const IloGeneratorArray& copy)
    :IloGeneratorArrayBase(copy) {}
  IloGeneratorArray(const IloEnv env, IloInt n = 0)
    :IloGeneratorArrayBase(env, n) {}
};


#endif
