// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_iloforalliH
#define __CONCERT_iloforalliH


#if !defined(__CONCERT_iloforallbaseH)
# include <ilconcert/iloforallbase.h>
#endif
#if !defined(__CONCERT_ilocollectionH)
# include <ilconcert/ilocollection.h>
#endif
#if !defined(__CONCERT_ilotuplecollectioniH)
# include <ilconcert/ilotuplecollectioni.h>
#endif
#if !defined(__CONCERT_ilotupleH)
# include <ilconcert/ilotuple.h>
#endif
#if !defined(__CONCERT_iloexpressionH)
# include <ilconcert/iloexpression.h>
#endif


class IloIntCollectionExprArg;

class IloNumCollectionExprArg;

class IloSymbolCollectionExprArg;

class IloIntGeneratorI;
class IloNumGeneratorI;
class IloAnyGeneratorI;
class IloModelEvaluatorI;
class IloIntSetVarI;
class IloIntIndex;
class IloNumIndex;
class IloSymbolIndex;
class IloTupleIndex;
class IloTuplePattern;
class IloTuplePatternI;
class IloGeneratorI;
#define IloSymbolGeneratorI IloAnyGeneratorI
#define IloTupleGeneratorI IloAnyGeneratorI

/////////////////////////////////////////////////////////////////////////////
//   Formal Indexes
/////////////////////////////////////////////////////////////////////////////

class IloIntSubMapExpr;
class IloIntSubMapExprI;
class IloNumSubMapExpr;
class IloNumSubMapExprI;
class IloSymbolSubMapExpr;
class IloSymbolSubMapExprI;
class IloTupleSchemaI;


//-------------------------------------------
class IloIntIndexI : public IloIntExprI {
  ILOEXTRDECL
  void visitSubExtractables(IloExtractableVisitor* v);
  virtual IloBool hasNongroundType() const;
public:
  IloIntIndexI(IloEnvI* env, const char* name=0);
  virtual ~IloIntIndexI();
  IloInt getKey() const;
  IloIntGeneratorI* within(IloIntCollectionI* coll) const;
  IloIntGeneratorI* within(IloIntSetVarI* var) const;

  IloIntGeneratorI* within(IloIntCollectionExprArgI* expr) const;

  IloNum eval(const IloAlgorithm alg) const;

  ILOEXTROTHERDECL
};

//-------------------------------------------
class IloNumIndexI : public IloNumExprI {
  ILOEXTRDECL
  void visitSubExtractables(IloExtractableVisitor* v);
  virtual IloBool hasNongroundType() const;
public:
  IloNumIndexI(IloEnvI* env, const char* name=0);
  virtual ~IloNumIndexI();
  IloInt getKey() const;
  IloNumGeneratorI* within(IloNumCollectionI* coll) const;

  IloNumGeneratorI* within(IloNumCollectionExprArgI* expr) const;

  IloNum eval(const IloAlgorithm alg) const;

  ILOEXTROTHERDECL
};

//-------------------------------------------
class IloAnyIndexI : public IloAnyExprI {
  ILOEXTRDECL
  void visitSubExtractables(IloExtractableVisitor* v);
  virtual IloBool hasNongroundType() const;
public:
  IloAnyIndexI(IloEnvI* env, const char* name=0);
  virtual ~IloAnyIndexI();
  IloInt getKey() const;
  virtual void display(ILOSTD(ostream)& out) const;
  virtual void errorNotSubstituted() const=0;
};

//-------------------------------------------
class IloSymbolIndexI : public IloAnyIndexI {
  ILOEXTRDECL
  IloExtractableI* makeClone(IloEnvI* env) const;
public:
  IloSymbolIndexI(IloEnvI* env, const char* name=0);
  virtual IloEncoderI* getEncoder() const;
  IloSymbolGeneratorI* within(IloAnyCollectionI* coll) const;

  IloSymbolGeneratorI* within(IloSymbolCollectionExprArgI* expr) const;

  virtual void errorNotSubstituted() const;
};

//-------------------------------------------
class IloTupleIndexI : public IloAnyIndexI {
protected:
  IloBool _isKey;
  ILOEXTRDECL
  IloExtractableI* makeClone(IloEnvI* env) const;
public:

  IloTupleIndexI(IloEnvI* env, const char* name=0);
  virtual IloEncoderI* getEncoder() const;
  IloTupleGeneratorI* within(IloTupleSetI* coll) const;
  IloTupleGeneratorI* within(IloTupleSetExprArgI* expr) const;

  virtual void checkCompatibility(IloTupleSchemaI*) const;
  virtual void errorNotSubstituted() const;
  virtual IloAnyExprI* makeImage() const;
  virtual void setKeyProperty(IloBool flag) { _isKey = flag; }
  IloBool isKey() const{ return _isKey; }
  virtual void display(ILOSTD(ostream)& out) const;
};

//-------------------------------------------
class IloTupleIndexImageI : public IloTupleIndexI {
  IloTupleIndexI* _object;
  ILOEXTRDECL
  IloExtractableI* makeClone(IloEnvI* env) const;
#ifdef OPL_1748
  //void visitSubExtractables(IloExtractableVisitor* v);
#endif
public:
  IloTupleIndexImageI(IloEnvI* env, IloTupleIndexI* object, const char* name=0);
  virtual IloAnyExprI* makeImage() const;
  virtual IloTupleIndexI* getTupleIndex() const {return _object;}
#ifdef OPL_1748
  IloTupleIndexI* getPrimaryTupleIndex() const{
	  IloTupleIndexI* ret = _object;
	  while(ret->getTypeInfo() == IloTupleIndexImageI::GetTypeInfo()){
		  ret = ((IloTupleIndexImageI*)ret)->getTupleIndex();
	  }
	  return ret;
  }
#endif
  virtual void display(ILOSTD(ostream)& out) const;
  virtual void setKeyProperty(IloBool flag) { 
	  _isKey = flag; 
	  _object->setKeyProperty(flag);
  }
};

//-------------------------------------------
class IloIntTupleCellExprI : public IloIntExprI {
  ILOEXTRDECL
private:
  IloTupleExprI* _tuple;
  IloInt _colIndex;
  IloSymbolI* _colName;
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntTupleCellExprI(IloEnvI* env,
			IloTupleExprI* tuple, IloInt colIndex);
  IloIntTupleCellExprI(IloEnvI* env,
			IloTupleExprI* tuple, IloSymbolI* colName);
  virtual ~IloIntTupleCellExprI();
  IloTupleExprI* getTuple() const { return _tuple; }
  IloInt getColumnIndex() const { return _colIndex; }
  IloSymbolI* getColumnName() const { return _colName; }
  IloNum eval(const IloAlgorithm alg) const;

  ILOEXTROTHERDECL
};

//-------------------------------------------
class IloNumTupleCellExprI : public IloNumExprI {
  ILOEXTRDECL
private:
  IloTupleExprI* _tuple;
  IloInt _colIndex;
  IloSymbolI* _colName;
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumTupleCellExprI(IloEnvI* env,
			IloTupleExprI* tuple, IloInt colIndex);
  IloNumTupleCellExprI(IloEnvI* env,
			IloTupleExprI* tuple, IloSymbolI* colName);
  virtual ~IloNumTupleCellExprI();
  IloTupleExprI* getTuple() const { return _tuple; }
  IloInt getColumnIndex() const { return _colIndex; }
  IloSymbolI* getColumnName() const { return _colName; }
  IloNum eval(const IloAlgorithm alg) const;

  ILOEXTROTHERDECL
};

//-------------------------------------------
class IloSymbolTupleCellExprI : public IloSymbolExprI {
  ILOEXTRDECL
private:
  IloTupleExprI* _tuple;
  IloInt _colIndex;
  IloSymbolI* _colName;
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloSymbolTupleCellExprI(IloEnvI* env,
			   IloTupleExprI* tuple, IloInt colIndex);
  IloSymbolTupleCellExprI(IloEnvI* env,
			   IloTupleExprI* tuple, IloSymbolI* colName);
  virtual ~IloSymbolTupleCellExprI();
  virtual IloEncoderI* getEncoder() const;
  IloTupleExprI* getTuple() const { return _tuple; }
  IloInt getColumnIndex() const { return _colIndex; }
  IloSymbolI* getColumnName() const { return _colName; }

  ILOEXTROTHERDECL
};

//-------------------------------------------
class IloTupleTupleCellExprI : public IloTupleExprI {
  ILOEXTRDECL
private:
  IloBool _isKey;
  IloTupleExprI* _tuple;
  IloInt _colIndex;
  IloSymbolI* _colName;
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTupleTupleCellExprI(IloEnvI* env,
			  IloTupleExprI* tuple, IloInt colIndex);
  IloTupleTupleCellExprI(IloEnvI* env,
			  IloTupleExprI* tuple, IloSymbolI* colName);
  virtual ~IloTupleTupleCellExprI();
  IloTupleExprI* getTuple() const { return _tuple; }
  IloInt getColumnIndex() const { return _colIndex; }
  IloSymbolI* getColumnName() const { return _colName; }
  virtual IloEncoderI* getEncoder() const {
    return getEnv()->getTupleEncoder();
  }
  virtual void setKeyProperty(IloBool flag) { _isKey = flag; }
  IloBool isKey() const{ return _isKey; }
  ILOEXTROTHERDECL
};

//-------------------------------------------
class IloTuplePatternItem {
	friend class IloTuplePattern;
	friend class IloTuplePatternI;
public:
  //1: Extractable ?
  //2: Index ?
  //3: Int ?
  //4: Num ?
  //5: Symbol ?
  //6: Tuple ?
  //7: Pattern ?
  enum Type {
    //Anonymous=0,
    IntConst =4,
    IntExpr  =5,
    IntIndex =7,
	NumConst =8,
    NumExpr  =9,
    NumIndex =11,
	SymbolConst=16,
	SymbolExpr = 17,
    SymbolIndex=19,
    TupleConst=32,
    TupleExpr = 33,
	TupleIndex=35,
    TuplePattern=67,
	IntCollectionConst=128,
	IntCollectionExpr=129,
	IntCollectionIndex=131,
	NumCollectionConst=256,
	NumCollectionExpr=257,
	NumCollectionIndex=259,
	SymbolCollectionConst=512,
	SymbolCollectionExpr=513,
	SymbolCollectionIndex=515
  };
private:
  Type   _type;
  union {
    IloInt _int;
    IloNum _num;
    IloAny _any;
  } _data;
  IloTuplePatternItem(Type t, IloAny e)
    :_type(t) { _data._any = e;}
public:
  IloTuplePatternItem(const IloTuplePatternItem& i);

  IloTuplePatternItem(IloIntExprArg x);
  IloTuplePatternItem(IloIntIndex x);
  IloTuplePatternItem(IloInt x);
  IloTuplePatternItem(IloNumExprArg x);
  IloTuplePatternItem(IloNumIndex x);
  IloTuplePatternItem(IloNum x);
  IloTuplePatternItem(IloSymbolExprArg x);
  IloTuplePatternItem(IloSymbolIndex x);
  IloTuplePatternItem(const char* x);
  IloTuplePatternItem(IloSymbol x);
  IloTuplePatternItem(IloTupleIndex x);
  IloTuplePatternItem(IloTupleExprArg x);
  IloTuplePatternItem(IloTuple x);
  IloTuplePatternItem(IloTuplePattern x);

  IloTuplePatternItem(IloIntCollectionExprArg x);
  IloTuplePatternItem(IloIntCollectionIndex x);
  IloTuplePatternItem(IloIntCollection x);
  IloTuplePatternItem(IloNumCollectionExprArg x);
  IloTuplePatternItem(IloNumCollectionIndex x);
  IloTuplePatternItem(IloNumCollection x);
  IloTuplePatternItem(IloSymbolCollectionExprArg x);
  IloTuplePatternItem(IloSymbolCollectionIndex x);
  IloTuplePatternItem(IloAnyCollection x);

  ~IloTuplePatternItem() {}
  Type getType() const { return _type; }
  IloTuplePatternItem getClone(IloEnvI* env);
  void operator= (const IloTuplePatternItem& i);
  IloBool operator== (const IloTuplePatternItem& i);
  IloBool isConstant() const { return !(_type & 1); }
  IloBool isExtractable() const { return (_type & 1); }
  IloBool isIndex() const { return (_type & 2); }
  IloBool isInt() const { return (_type & 4); }
  IloBool isNum() const { return (_type & 8); }
  IloBool isSymbol() const { return (_type & 16); }
  IloBool isTuple() const { return (_type & 32); }
  IloBool isPattern() const { return (_type & 64); }

  IloBool isIntCollection() const { return (_type & 128); }
  IloBool isNumCollection() const { return (_type & 256); }
  IloBool isSymbolCollection() const { return (_type & 512); }

  IloBool isAny() const{ return (!isInt() && !isNum());}
  IloInt asInt() const {
    IloAssert(_type == IntConst, "Pattern Item is not an integer.");
    return _data._int;
  }
  IloNum asNum() const {
    IloAssert(_type == NumConst, "Pattern Item is not a double.");
    return _data._num;
  }
  const char* asSymbol() const {
    IloAssert(_type == SymbolConst, "Pattern Item is not a symbol.");
    return (const char*)_data._any;
  }
  IloAny asAny() const{
	IloAssert((_type != NumConst) && (_type != IntConst), "Pattern Item can not be used as an IloAny");
	return _data._any;
  }
  IloIntCollectionI* asIntCollection() const {
    IloAssert(_type == IntCollectionConst, "Pattern Item is not a collection of int.");
    return (IloIntCollectionI*)_data._any;
  }
  IloNumCollectionI* asNumCollection() const {
    IloAssert(_type == NumCollectionConst, "Pattern Item is not a collection of num.");
    return (IloNumCollectionI*)_data._any;
  }
  IloAnyCollectionI* asSymbolCollection() const {
    IloAssert(_type == SymbolCollectionConst, "Pattern Item is not a collection of symbol.");
    return (IloAnyCollectionI*)_data._any;
  }

  IloTupleI* asTuple() const {
    IloAssert(_type == TupleConst, "Pattern Item is not a tuple.");
    return (IloTupleI*)_data._any;
  }
  IloExtractableI* asExtractable() const {
    IloAssert(isExtractable(), "Pattern Item is not an extractable.");
    return (IloExtractableI*)_data._any;
  }
  IloIntExprI* asIntExpr() const {
    IloAssert(isInt() && isExtractable(), "Pattern Item is not an integer expression.");
    return (IloIntExprI*)_data._any;
  }
  IloNumExprI* asNumExpr() const {
    IloAssert(isNum() && isExtractable(), "Pattern Item is not a numeric expression.");
    return (IloNumExprI*)_data._any;
  }
  IloIntCollectionExprI* asIntCollectionExpr() const {
    IloAssert(isIntCollection() && isExtractable(), "Pattern Item is not an intcollection expression.");
    return (IloIntCollectionExprI*)_data._any;
  }
  IloNumCollectionExprI* asNumCollectionExpr() const {
    IloAssert(isNumCollection() && isExtractable(), "Pattern Item is not a numcollection expression.");
    return (IloNumCollectionExprI*)_data._any;
  }
  IloSymbolCollectionExprI* asSymbolCollectionExpr() const {
    IloAssert(isSymbolCollection() && isExtractable(), "Pattern Item is not a numcollection expression.");
    return (IloSymbolCollectionExprI*)_data._any;
  }
  IloTuplePatternI* asPattern() const {
    IloAssert(_type == TuplePattern, "Pattern Item is not a tuple pattern.");
    return (IloTuplePatternI*)_data._any;
  }
  IloTupleIndexI* asTupleIndex() const {
    IloAssert(_type == TupleIndex, "Pattern Item is not a tuple index.");
    return (IloTupleIndexI*)_data._any;
  }
  IloTupleTupleCellExprI* asTupleTupleCellExpr() const{
    //IloAssert(_type == TupleIndex, "Pattern Item is not a tuple index.");  ?????
    return (IloTupleTupleCellExprI*)_data._any;
  }
  void display(ILOSTD(ostream)& out) const;
#ifdef OPL_1748
  IloMapItem asMapItem();
#endif
};
typedef IloArray<IloTuplePatternItem> IloTuplePatternItemArrayBase;

class IloTuplePatternItemArray : public IloTuplePatternItemArrayBase {
public:
  typedef IloDefaultArrayI ImplClass;
  IloTuplePatternItemArray(IloDefaultArrayI* i=0)
    :IloTuplePatternItemArrayBase(i) {}
  IloTuplePatternItemArray(const IloTuplePatternItemArray& copy)
    :IloTuplePatternItemArrayBase(copy) {}
  IloTuplePatternItemArray(const IloEnv env, IloInt n=0)
    :IloTuplePatternItemArrayBase(env, n) {}
  IloTuplePatternItemArray(const IloMemoryManager env, IloInt n=0)
    :IloTuplePatternItemArrayBase(env, n) {}
};

//-------------------------------------------
class IloTuplePatternI : public IloTupleIndexI {
  friend class IloTupleSet;
  ILOEXTRDECL
public:
  enum Status {
    ConstPattern=0,
    ExprPattern=1,
    UndefPattern=2
  };
private:
  Status _status;
private:
  IloInt _arity;
  IloTuplePatternItem* _items;
  IloTupleSchemaI* _schema;
  void visitSubExtractables(IloExtractableVisitor* v);
  IloBool isPatternConst();
public:
	IloBool hasSchema() const{ return _schema!=0;}
	IloTupleSchemaI* getSchema() const { return _schema; }
	void setSchema(IloTupleSchemaI* schema){ _schema = schema; }

  //void checkKeyProperty() const;
  virtual void checkCompatibility(IloTupleSchemaI* schema) const;
  void checkColumnCompatibility(IloTuplePatternItem* item,
				IloColumnDefinitionI* col, IloInt pos) const;
  void lockSubExpr();
  void releaseSubExpr(IloEnvI* env);
  IloTuplePatternI(IloEnvI* env, IloInt size, IloTuplePatternItem* items);
  virtual ~IloTuplePatternI();  IloInt getArity() const { return _arity; }
  IloTuplePatternItem* getItems() const { return _items; }
  IloTuplePatternItem* getItem(IloInt i) const {
    IloAssert(i >= 0, "Index out of bound.");
    IloAssert(i < _arity, "Index out of bound.");
    return &_items[i];
  }
  IloExtractableI* getExtractable(IloInt i) const {
    IloAssert(i >= 0, "Index out of bound.");
    IloAssert(i < _arity, "Index out of bound.");
    return _items[i].asExtractable();
  }
  IloInt getIndexKey(IloInt i) const;
  IloTuplePatternI* copy() const;
  IloTuplePatternI(IloEnvI* env, Status status, IloInt size, IloTuplePatternItem* items);
  Status getStatus() const { 
	  return _status; 
  };
  //void setStatus(Status status) { 
	 // _status = status; 
  //};
  IloBool isConst() const { 
	  return !_status; 
  }
  IloBool isExpr() const { return (_status & 1); }
  IloTupleSchemaI* buildSchema() const;
  ILOEXTROTHERDECL
  virtual void setKeyProperty(IloBool flag);
};

/////////////////////////////////////////////////////////////////////////////
//   Generators
/////////////////////////////////////////////////////////////////////////////

//-------------------------------------------
class IloIntGeneratorI : public IloGeneratorI {
  ILOEXTRDECL
private:
  IloIntIndexI* _x;
protected:
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntGeneratorI(IloEnvI* env, IloIntIndexI* x);
  virtual ~IloIntGeneratorI();
  IloInt getIndexKey() const;
  IloIntIndexI* getIndex() const { return _x; }
  virtual IloDataIterator* iterator() const{
	  throw IloNotSupportedOperation("IloIntGeneratorI::iterator");
	  ILOUNREACHABLE(return 0;)
  }
  virtual IloDiscreteDataCollectionI* getDiscreteDataCollection() const {
	  throw IloNotSupportedOperation("IloIntGeneratorI::getDiscreteDataCollection");
	  ILOUNREACHABLE(return 0;)
  }
};

//-------------------------------------------
class IloIntCollectionGeneratorI : public IloIntGeneratorI {
  ILOEXTRDECL
private:
  IloIntCollectionI* _coll;
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntCollectionGeneratorI(IloEnvI* env,
			     IloIntIndexI* x,
			     IloIntCollectionI* coll);
  virtual ~IloIntCollectionGeneratorI();
  IloIntCollectionI* getCollection() const { return _coll; }
  virtual IloDataIterator* iterator() const{
	  return getCollection()->iterator();
  }
  virtual IloDiscreteDataCollectionI* getDiscreteDataCollection() const {
	  return _coll;
  }
  virtual IloBool generatesDuplicates() const;
  ILOEXTROTHERDECL
};

//-------------------------------------------
class IloIntSetVarGeneratorI : public IloIntGeneratorI {
  ILOEXTRDECL
private:
  IloIntSetVarI* _var;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntSetVarGeneratorI(IloEnvI* env,
			 IloIntIndexI* x,
			 IloIntSetVarI* var);
  virtual ~IloIntSetVarGeneratorI();
  virtual IloBool isOpen() const;
  IloIntSetVarI* getSetVar() const { return _var; }
  virtual IloBool generatesDuplicates() const;
  ILOEXTROTHERDECL
};

//-------------------------------------------
class IloNumGeneratorI : public IloGeneratorI {
  ILOEXTRDECL
private:
  IloNumIndexI* _x;
protected:
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumGeneratorI(IloEnvI* env, IloNumIndexI* x);
  virtual ~IloNumGeneratorI();
  IloInt getIndexKey() const;
  IloNumIndexI* getIndex() const { return _x; }
  virtual IloDataIterator* iterator() const{
	  throw IloNotSupportedOperation("IloNumGeneratorI::iterator");
	  ILOUNREACHABLE(return 0;)
  }
  virtual IloDiscreteDataCollectionI* getDiscreteDataCollection() const {
	  throw IloNotSupportedOperation("IloNumGeneratorI::getDiscreteDataCollection");
	  ILOUNREACHABLE(return 0;)
  }
};

//-------------------------------------------
class IloNumCollectionGeneratorI : public IloNumGeneratorI {
  ILOEXTRDECL
private:
  IloNumCollectionI* _coll;
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumCollectionGeneratorI(IloEnvI* env,
			     IloNumIndexI* x,
			     IloNumCollectionI* coll);
  virtual ~IloNumCollectionGeneratorI();
  IloNumCollectionI* getCollection() const { return _coll; }
  virtual IloDataIterator* iterator() const{
	  return getCollection()->iterator();
  }
  virtual IloDiscreteDataCollectionI* getDiscreteDataCollection() const {
	  return _coll;
  }
  virtual IloBool generatesDuplicates() const;
  ILOEXTROTHERDECL
};

//-------------------------------------------
class IloAnyGeneratorI : public IloGeneratorI {
  ILOEXTRDECL
private:
  IloAnyIndexI* _x;
protected:
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloAnyGeneratorI(IloEnvI* env, IloAnyIndexI* x);
  virtual ~IloAnyGeneratorI();
  IloInt getIndexKey() const;
  IloAnyIndexI* getIndex() const { return _x; }
  virtual IloDataIterator* iterator() const{
	  throw IloNotSupportedOperation("IloAnyGeneratorI::iterator");
	  ILOUNREACHABLE(return 0;)
  }
  virtual IloDiscreteDataCollectionI* getDiscreteDataCollection() const {
	  throw IloNotSupportedOperation("IloAnyGeneratorI::getDiscreteDataCollection");
	  ILOUNREACHABLE(return 0;)
  }
};

//-------------------------------------------
class IloSymbolCollectionGeneratorI : public IloSymbolGeneratorI {
  ILOEXTRDECL
private:
  IloAnyCollectionI* _coll;
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloSymbolCollectionGeneratorI(IloEnvI* env,
				IloSymbolIndexI* x,
				IloAnyCollectionI* coll);
  IloAnyCollectionI* getCollection() const { return _coll; }
  virtual IloBool generatesDuplicates() const;
  virtual IloDataIterator* iterator() const{
	  return getCollection()->iterator();
  }
  virtual IloDiscreteDataCollectionI* getDiscreteDataCollection() const {
	  return _coll;
  }
  ILOEXTROTHERDECL
};

//-------------------------------------------
class IloTupleSetGeneratorI : public IloTupleGeneratorI {
  ILOEXTRDECL
private:
  IloTupleSetI* _coll;
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTupleSetGeneratorI(IloEnvI* env,
			       IloTupleIndexI* x,
			       IloTupleSetI* coll);
  IloTupleSetI* getCollection() const { return _coll; }
  virtual IloDataIterator* iterator() const{
	  return getCollection()->iterator();
  }
  virtual IloBool generatesDuplicates() const;
  virtual IloDiscreteDataCollectionI* getDiscreteDataCollection() const {
	  return _coll;
  }

  ILOEXTROTHERDECL
};

class IloTupleSetExprGeneratorI : public IloTupleGeneratorI {
  ILOEXTRDECL
private:
  IloTupleSetExprI* _coll;
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTupleSetExprGeneratorI(IloEnvI* env,
			       IloTupleIndexI* x,
			       IloTupleSetExprI* coll);
  IloTupleSetExprI* getCollection() const { return _coll; }
  virtual IloBool generatesDuplicates() const;
  virtual IloDiscreteDataCollectionI* getDiscreteDataCollection() const {
	  throw IloNotSupportedOperation("IloTupleSetExprGeneratorI::getDiscreteDataCollection");
	  ILOUNREACHABLE(return 0;)
  }

  ILOEXTROTHERDECL
};

/////////////////////////////////////////////////////////////////////////////
//   Comprehension
/////////////////////////////////////////////////////////////////////////////
class IloMapIndexArray;
class IloComprehensionI : public IloExtractableI {
  ILOEXTRDECL
private:
  IloInt _nGenerators;
  IloGeneratorI** _generators;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  virtual IloBool hasNongroundType() const;
public:
  IloComprehensionI(IloEnvI* env, IloGeneratorI* g1);
  IloComprehensionI(IloEnvI* env, IloGeneratorArray generators);
  IloComprehensionI(IloEnvI* env, IloInt n, IloGeneratorI** generators);

  virtual ~IloComprehensionI();
  IloInt getNbGenerators() const { return _nGenerators; }
  IloGeneratorI** getGenerators() const { return _generators; }
  IloMapIndexArray makeMapIndexArray() const;
  ILOEXTROTHERDECL
};

class IloExtendedComprehensionI : public IloComprehensionI {
  ILOEXTRDECL
private:
  IloExtractableI* _extent;
  enum ExtentType {
    Other,
    IntExpr,
    NumExpr,
    AnyExpr
  };
  ExtentType _extentType;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  void lockExtentExpr(IloExtractableI* e);
  void releaseExtentExpr(IloExtractableI* e);
public:
  IloExtendedComprehensionI(IloEnvI* env, IloGeneratorI* g1, IloExtractableI* e);
  IloExtendedComprehensionI(IloEnvI* env, IloGeneratorArray gen, IloExtractableI* e);
  IloExtendedComprehensionI(IloEnvI* env, IloInt n,IloGeneratorI** gen,IloExtractableI* e);
  virtual ~IloExtendedComprehensionI();
  IloExtractableI* getExtent() const { return _extent; }

  ILOEXTROTHERDECL
};

/////////////////////////////////////////////////////////////////////////////
// For-all
/////////////////////////////////////////////////////////////////////////////
class IloForAllI : public IloConstraintI {
  ILOEXTRDECL
  friend class IloEvalForallI;
private:
  IloComprehensionI* _comp;
  IloConstraintI*    _ct;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  virtual IloBool hasNongroundType() const;
#ifndef ILCFORALL
  virtual void whenAddedToCP(const IloAlgorithm alg, IloAny ptr) const;
#endif //ifndef ILCFORALL
public:
  IloForAllI(IloEnvI* env,
	     IloComprehensionI* comp,
	     IloConstraintI* ct,
	     const char* name=0);
  virtual ~IloForAllI();
  virtual IloInt expandTo(IloModelI* m, const char* propertyName=0, const char* propertyValue=0) const;
  IloComprehensionI* getComprehension() const { return _comp; }
  IloInt getNbGenerators() const { return _comp->getNbGenerators(); }
  IloGeneratorI** getGenerators() const { return _comp->getGenerators(); }
  IloConstraintI* getConstraint() const { return _ct; }
  IloConstraintI* findConstraint(const char* propertyLabel, const char* propertyValue) const;
  IloConstraintI* focusOnConstraint(const char* propertyLabel, const char* propertyValue) const;
  ILOEXTROTHERDECL
};



enum IloAggregateOperator {
  IloAggregateSum,
  IloAggregateProd,
  IloAggregateMin,
  IloAggregateMax, 
  IloAggregateOr,
  IloAggregateAnd
};
const char* IloPrintOperator(IloAggregateOperator);

class IloAggregateConstraintI : public IloConstraintI {
  ILOEXTRDECL
private:
  IloComprehensionI*   _comp;
  IloConstraintI*   _ct;
  IloAggregateOperator  _op;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  virtual IloBool hasNongroundType() const;
public:
  IloAggregateConstraintI(IloEnvI* env, IloComprehensionI* comp, IloConstraintI* ct, IloAggregateOperator op, 
	  const char* name=0);
  virtual ~IloAggregateConstraintI();
  virtual IloInt expandTo(IloModelI* m, const char* propertyName=0, const char* propertyValue=0) const;
  IloComprehensionI* getComprehension() const { return _comp; }
  IloInt getNbGenerators() const { return _comp->getNbGenerators(); }
  IloGeneratorI** getGenerators() const { return _comp->getGenerators(); }
  IloConstraintI* getConstraint() const { return _ct; }
  IloAggregateOperator getOperator() const { return _op; }
  ILOEXTROTHERDECL
};


/////////////////////////////////////////////////////////////////////////////
// Aggregation operators
/////////////////////////////////////////////////////////////////////////////

class IloIntAggregateExprI : public IloIntExprI {
  ILOEXTRDECL
private:
  IloComprehensionI* _comp;
  IloAggregateOperator _op;
  IloIntExprI* _expr;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  virtual IloBool hasNongroundType() const;
public:
  IloIntAggregateExprI(IloEnvI* env,
		       IloComprehensionI* comp,
		       IloAggregateOperator op,
		       IloIntExprI* expr);
  virtual ~IloIntAggregateExprI();
  virtual IloInt expandTo(IloModelI* m, const char* propertyName=0, const char* propertyValue=0) const {
     return IloIntExprI::expandTo(m, propertyName, propertyValue);
  }
  virtual IloInt expandTo(IloNumExprI* e) const;
  IloComprehensionI* getComprehension() const { return _comp; }
  IloInt getNbGenerators() const { return _comp->getNbGenerators(); }
  IloGeneratorI** getGenerators() const { return _comp->getGenerators(); }
  IloAggregateOperator getOperator() const { return _op; }
  IloIntExprI* getExpr() const { return _expr; }
  IloNum eval(const IloAlgorithm alg) const;
  ILOEXTROTHERDECL
};

class IloNumAggregateExprI : public IloNumExprI {
  ILOEXTRDECL
private:
  IloComprehensionI* _comp;
  IloAggregateOperator _op;
  IloNumExprI* _expr;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
  virtual IloBool hasNongroundType() const;
public:
  IloNumAggregateExprI(IloEnvI* env,
		       IloComprehensionI* comp,
		       IloAggregateOperator op,
		       IloNumExprI* expr);
  virtual ~IloNumAggregateExprI();
  virtual IloInt expandTo(IloModelI* m, const char* propertyName=0, const char* propertyValue=0) const {
     return IloNumExprI::expandTo(m, propertyName, propertyValue);
  }
  virtual IloInt expandTo(IloNumExprI* e) const;
  IloComprehensionI* getComprehension() const { return _comp; }
  IloInt getNbGenerators() const { return _comp->getNbGenerators(); }
  IloGeneratorI** getGenerators() const { return _comp->getGenerators(); }
  IloAggregateOperator getOperator() const { return _op; }
  IloNumExprI* getExpr() const { return _expr; }
  IloNum eval(const IloAlgorithm alg) const;
  ILOEXTROTHERDECL
};

/////////////////////////////////////////////////////////////////////////////
// Aggregate piecewise linear function
/////////////////////////////////////////////////////////////////////////////
class IloAggregatePwlExprI : public IloNumExprI {
  ILOEXTRDECL
private:
  IloNumExprI* _node;
  IloComprehensionI* _comp;
  virtual IloBool hasNongroundType() const;
public:
  IloAggregatePwlExprI(IloEnvI* env,
		       IloNumExprI* node, IloComprehensionI* comp);
  virtual ~IloAggregatePwlExprI();
  virtual IloInt expandTo(IloModelI* m, const char* propertyName=0, const char* propertyValue=0) const {
     return IloNumExprI::expandTo(m, propertyName, propertyValue);
  }
  virtual IloInt expandTo(IloNumExprI* e) const = 0;
  IloNumExprI* getNode() const { return _node; }
  IloComprehensionI* getComprehension() const { return _comp; }
  IloInt getNbGenerators() const { return _comp->getNbGenerators(); }
  IloGeneratorI** getGenerators() const { return _comp->getGenerators(); }
};

class IloAggregateSlopesPwlExprI : public IloAggregatePwlExprI {
  ILOEXTRDECL
private:
  IloNumExprI* _point;
  IloNumExprI* _slope;
  IloNumExprI* _lastSlope;
  IloNumExprI* _x;
  IloNumExprI* _y;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloAggregateSlopesPwlExprI(IloEnvI* env,
			     IloNumExprI* node,
			     IloComprehensionI* comp,
			     IloNumExprI* point,
			     IloNumExprI* slope,
			     IloNumExprI* lastSlope,
			     IloNumExprI* x,
			     IloNumExprI* y);
  virtual ~IloAggregateSlopesPwlExprI();
  virtual IloInt expandTo(IloModelI* m, const char* propertyName=0, const char* propertyValue=0) const {
     return IloAggregatePwlExprI::expandTo(m, propertyName, propertyValue);
  }
  virtual IloInt expandTo(IloNumExprI* e) const;
  IloNumExprI* getPoint() const { return _point; }
  IloNumExprI* getSlope() const { return _slope; }
  IloNumExprI* getLastSlope() const { return _lastSlope; }
  IloNumExprI* getX() const { return _x; }
  IloNumExprI* getY() const { return _y; }
  IloNum eval(const IloAlgorithm alg) const;
  ILOEXTROTHERDECL
};

class IloAggregatePointsPwlExprI : public IloAggregatePwlExprI {
  ILOEXTRDECL
private:
  IloNumExprI* _x;
  IloNumExprI* _y;
  IloNumExprI* _firstSlope;
  IloNumExprI* _lastSlope;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloAggregatePointsPwlExprI(IloEnvI* env,
			     IloNumExprI* node,
			     IloNumExprI* firstSlope,
			     IloComprehensionI* comp,
			     IloNumExprI* x, IloNumExprI* y,
			     IloNumExprI* lastSlope);
  virtual ~IloAggregatePointsPwlExprI();
  virtual IloInt expandTo(IloModelI* m, const char* propertyName=0, const char* propertyValue=0) const {
     return IloAggregatePwlExprI::expandTo(m, propertyName, propertyValue);
  }
  virtual IloInt expandTo(IloNumExprI* e) const;
  IloNumExprI* getX() const { return _x; }
  IloNumExprI* getY() const { return _y; }
  IloNumExprI* getFirstSlope() const { return _firstSlope; }
  IloNumExprI* getLastSlope()  const { return _lastSlope; }
  IloNum eval(const IloAlgorithm alg) const;
  ILOEXTROTHERDECL
};



/////////////////////////////////////////////////////////////////////////////
//   Aggregate Sets
/////////////////////////////////////////////////////////////////////////////
class IloIntAggregateSetI : public IloIntSetI {
  ILORTTIDECL
private:
  IloExtendedComprehensionI* _comp;
public:
  IloIntAggregateSetI(IloExtendedComprehensionI* comp);
  IloIntAggregateSetI(IloEnvI* env, IloIntExprI* e, IloGeneratorI* g);
  IloIntAggregateSetI(IloEnvI* env, IloIntExprI* e,
		      IloGeneratorArray generators);
  virtual ~IloIntAggregateSetI();
  IloIntExprI* getBody() const { return (IloIntExprI*)_comp->getExtent(); }
  IloComprehensionI* getComprehension() const { return _comp; }
  virtual void display(ILOSTD(ostream)& out) const;
  virtual void displayContent(ILOSTD(ostream)& out) const;
};

class IloNumAggregateSetI : public IloNumSetI {
  ILORTTIDECL
private:
  IloExtendedComprehensionI* _comp;
public:
  IloNumAggregateSetI(IloExtendedComprehensionI* comp);

  IloNumAggregateSetI(IloEnvI* env, IloNumExprI* e, IloGeneratorI* g);
  IloNumAggregateSetI(IloEnvI* env, IloNumExprI* e,
		      IloGeneratorArray generators);
  virtual ~IloNumAggregateSetI();
  IloNumExprI* getBody() const { return (IloNumExprI*)_comp->getExtent(); }
  virtual void display(ILOSTD(ostream)& out) const;
  virtual void displayContent(ILOSTD(ostream)& out) const;
};

class IloSymbolAggregateSetI : public IloAnySetI {
  ILORTTIDECL
private:
  IloExtendedComprehensionI* _comp;
public:
  IloSymbolAggregateSetI(IloExtendedComprehensionI* comp);

  IloSymbolAggregateSetI(IloEnvI* env, IloSymbolExprI* e, IloGeneratorI* g);
  IloSymbolAggregateSetI(IloEnvI* env, IloSymbolExprI* e,
			 IloGeneratorArray generators);
  virtual ~IloSymbolAggregateSetI();
  IloSymbolIndexI* getBody() const { return (IloSymbolIndexI*)_comp->getExtent(); }
  virtual void display(ILOSTD(ostream)& out) const;
  virtual void displayContent(ILOSTD(ostream)& out) const;
  virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const {
    getEnv()->getSymbolEncoder()->display(out, getValue(index));
  }
};

class IloTupleAggregateSetI : public IloTupleSetI {
  ILORTTIDECL
private:
  IloExtendedComprehensionI* _comp;
public:
  IloTupleAggregateSetI(IloTupleSchemaI* schema, IloExtendedComprehensionI* comp);

  IloTupleAggregateSetI(IloTupleSchemaI* schema, 
	  IloIntArray names, IloArray<IloDiscreteDataCollectionI*> references, 
	  IloExtendedComprehensionI* comp);
  IloTupleAggregateSetI(IloEnvI* env, IloTupleSchemaI* schema,
			IloTupleIndexI* e,
			IloInt size, IloGeneratorI** generators);
  virtual ~IloTupleAggregateSetI();
  IloTupleIndexI* getBody() const { return (IloTupleIndexI*)_comp->getExtent(); }
  virtual void display(ILOSTD(ostream)& out) const;
  virtual void displayContent(ILOSTD(ostream)& out) const;
};

#endif
