// ---------------------------------------------------*- C++ -*--------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilomapH
#define __CONCERT_ilomapH

#if !defined(__CONCERT_iloforallH)
#include <ilconcert/iloforall.h>
#endif
#if !defined(__CONCERT_ilomapiH)
#include <ilconcert/ilomapi.h>
#endif
#if !defined(__CONCERT_ilocollexprH)
#include <ilconcert/ilocollexpr.h>
#endif

#define IloSymbolSetI IloAnySetI
#define TEST_VB

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//
// Unidimensional Maps
//
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

/////////////////////////////////////
//
// MACROs for subscript operators
//

// ----------------------------------------------------------------------
// IloXXXExprArg operator[](arg)
//     arg: IloIntExprArg, IloNumExprArg, IloSymbolExprArg,
//          IloTupleExprArg, IloTuplePattern
// ----------------------------------------------------------------------
#define DEFINE_EXPR_ACCESSORS(_exprType)\
  _exprType subscriptOp(IloIntExprArg index) const;\
  _exprType subscriptOp(IloNumExprArg index) const;\
  _exprType subscriptOp(IloSymbolExprArg index) const;\
  _exprType subscriptOp(IloTupleExprArg index) const;\
  _exprType subscriptOp(IloTuplePattern index) const;\
  _exprType operator[](IloIntExprArg index) const {\
    IloAssert (getImpl(), "Empty Handle in IloMap::operator[]");\
    IloAssert(getImpl()->isIndexerIntCompatible(),\
	      "IloMap::operator[]/get/set/at Indexer type mismatch");\
    return this->subscriptOp(index);\
  }\
  _exprType operator[](IloNumExprArg index) const {\
    IloAssert (getImpl(), "Empty Handle in IloMap::operator[]");\
    IloAssert(getImpl()->isIndexerNumCompatible(),\
	      "IloMap::operator[]/get/set/at Indexer type mismatch");\
    return this->subscriptOp(index);\
  }\
  _exprType operator[](IloSymbolExprArg index) const {\
    IloAssert (getImpl(), "Empty Handle in IloMap::operator[]");\
    IloAssert(getImpl()->isIndexerSymbolCompatible(),\
	      "IloMap::operator[]/get/set/at Indexer type mismatch");\
    return this->subscriptOp(index);\
  }\
  _exprType operator[](IloTupleExprArg index) const {\
    IloAssert (getImpl(), "Empty Handle in IloMap::operator[]");\
    IloAssert(getImpl()->isIndexerTupleCompatible(),\
	      "IloMap::operator[]/get/set/at Indexer type mismatch");\
    return this->subscriptOp(index);\
  }\
  _exprType operator[](IloTuplePattern index) const {\
    IloAssert (getImpl(), "Empty Handle in IloMap::operator[]");\
    IloAssert(getImpl()->isIndexerTupleCompatible(),\
	      "IloMap::operator[]/get/set/at Indexer type mismatch");\
    return this->subscriptOp(index);\
  }

#define REDEFINE_EXPR_ACCESSORS(_exprType)\
  _exprType operator[](IloIntExprArg index) const {\
    return super::operator[](index);\
  }\
  _exprType operator[](IloNumExprArg index) const {\
    return super::operator[](index);\
  }\
  _exprType operator[](IloSymbolExprArg index) const {\
    return super::operator[](index);\
  }\
  _exprType operator[](IloTupleExprArg index) const {\
    return super::operator[](index);\
  }\
  _exprType operator[](IloTuplePattern index) const {\
    return super::operator[](index);\
  }

#define REDEFINE_EXPR_ACCESSORS_CASTFROMIMPL(_exprType)\
  _exprType operator[](IloIntExprArg index) const {\
    return super::operator[](index).getImpl();\
  }\
  _exprType operator[](IloNumExprArg index) const {\
    return super::operator[](index).getImpl();\
  }\
  _exprType operator[](IloSymbolExprArg index) const {\
    return super::operator[](index).getImpl();\
  }\
  _exprType operator[](IloTupleExprArg index) const {\
    return super::operator[](index).getImpl();\
  }\
  _exprType operator[](IloTuplePattern index) const {\
    return super::operator[](index).getImpl();\
  }

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//
// Multidimensional Maps
//
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

/////////////////////////////////////
//
// MACROs for subscript operators
//
// ----------------------------------------------------------------------
// map[].elt(arg)
//     arg: IloInt, IloNum, IloSymbol, IloTuple, const char*
//          int, char short
// ----------------------------------------------------------------------
#define REDEFINE_ELT_MAP_ACCESSORS_BOUND(_eltType)\
  _eltType& eltAtAbsoluteIndex(IloInt i) const {\
      return (*(_eltType*)&(super::eltAtAbsoluteIndex(i)));\
  } \
  _eltType& fastEltAtAbsoluteIndex(IloInt i) const {\
      return (*(_eltType*)&(super::fastEltAtAbsoluteIndex(i)));\
  } \
  _eltType getAt(IloMapIndexArray indices) const {\
    return _eltType((name2(_eltType, I)*)super::getAt(indices).getImpl());\
  }\
  const _eltType get(IloInt index) const{\
    return _eltType((name2(_eltType, I)*)super::get(index).getImpl());\
  }\
  const _eltType get(IloNum index) const{\
    return _eltType((name2(_eltType, I)*)super::get(index).getImpl());\
  }\
  const _eltType get(const char* index) const{\
    return _eltType((name2(_eltType, I)*)super::get(index).getImpl());\
  }\
  const _eltType get(IloSymbol index) const{\
    return _eltType((name2(_eltType, I)*)super::get(index).getImpl());\
  }\
  const _eltType get(IloTuple index) const{\
    return _eltType((name2(_eltType, I)*)super::get(index).getImpl());\
  }


#define REDEFINE_SIMPLE_ELT_MAP_ACCESSORS_BOUND(_eltType)\
  _eltType& eltAtAbsoluteIndex(IloInt i) const {\
      return super::eltAtAbsoluteIndex(i);\
  } \
  _eltType& fastEltAtAbsoluteIndex(IloInt i) const {\
      return super::fastEltAtAbsoluteIndex(i);\
  } \
  const _eltType get(IloInt index) const{\
    return super::get(index);\
  }\
  const _eltType get(IloNum index) const{\
    return super::get(index);\
  }\
  const _eltType get(const char* index) const{\
    return super::get(index);\
  }\
  const _eltType get(IloSymbol index) const{\
    return super::get(index);\
  }\
  const _eltType get(IloTuple index) const{\
    return super::get(index);\
  }



// ----------------------------------------------------------------------
// subMap map[arg]
//     arg: IloInt, IloNum, IloSymbol, IloTuple, const char*
//          int, char short
// ----------------------------------------------------------------------
#define REDEFINE_SUBMAP_ACCESSORS_BOUND(_subMapType)\
  _subMapType getSub(IloNum index) const {\
    return _subMapType(super::getSub(index).getImpl());\
  }\
  _subMapType getSub(IloInt index) const {\
    return _subMapType(super::getSub(index).getImpl());\
  }\
  _subMapType getSub(IloTuple index) const {\
    return _subMapType(super::getSub(index).getImpl());\
  }\
  _subMapType getSub(const char* index) const {\
    return _subMapType(super::getSub(index).getImpl());\
  }\
  _subMapType operator[](IloInt index) const {\
    return _subMapType(super::operator[](index).getImpl());\
  }\
  _subMapType operator[](IloNum index) const {\
    return _subMapType(super::operator[](index).getImpl());\
  }\
  _subMapType operator[](const char* index) const {\
    return _subMapType(super::operator[](index).getImpl());\
  }\
  _subMapType operator[](IloSymbol index) const {\
    return _subMapType(super::operator[](index).getImpl());\
  }\
  _subMapType operator[](IloTuple index) const {\
    return _subMapType(super::operator[](index).getImpl());\
  }

#define REDEFINE_ACCESSORS_BOUND(_eltType, _subMapType)\
  REDEFINE_ELT_MAP_ACCESSORS_BOUND(_eltType)\
  REDEFINE_SUBMAP_ACCESSORS_BOUND(_subMapType)

#define REDEFINE_SIMPLE_ACCESSORS_BOUND(_eltType, _subMapType)\
  REDEFINE_SIMPLE_ELT_MAP_ACCESSORS_BOUND(_eltType)\
  REDEFINE_SUBMAP_ACCESSORS_BOUND(_subMapType)

/////////////////////////////////////
//
// IloMap - template part
//

typedef IloArray<IloDiscreteDataCollection> IloMapIndexer;

template <class Elt> class IloMap  {
  typedef IloMapI ImplClass;
public:
  typedef IloMapI* IloSubMapI;
private:
  IloMapI* _impl;
  inline IloSubMapI* subArray(IloInt i) const {
    return &((IloSubMapI*)getImpl()->getBase(i))[i&getImpl()->getMod()];
  }
  inline Elt* data(IloInt i) const {
    return &((Elt*)getImpl()->getBase(i))[i&getImpl()->getMod()];
  }
  inline void setData(IloInt i, Elt value) {
    *data(i) = value;
  }
protected:
  Elt& at(IloInt index) const {
    IloAssert (_impl, "Empty Handle in IloMap::get/set/at");
    IloTestAndRaise(1==getNbDim(),
		    "IloMap::get/set/at the map is not uni-dimensional.");
    IloInt i = _impl->getAbsoluteIndex(index);
    if (!(i>=0 && i<getSize()))
      throw IloMapOutOfBoundException(_impl, IloMapIndex(index));
    return *data(i);
  }

  Elt& at(IloNum index) const {
    IloAssert (_impl, "Empty Handle in IloMap::get/set/at");
    IloTestAndRaise(1==getNbDim(),
		    "IloMap::get/set/at the map is not uni-dimensional.");
    IloInt i = _impl->getAbsoluteIndex(index);
    if (!(i>=0 && i<_impl->getSize()))
      throw IloMapOutOfBoundException(_impl, IloMapIndex(index));
    return *data(i);
  }

  Elt& at(const char* index) const {
    IloAssert (_impl, "Empty Handle in IloMap::get/set/at");
    IloTestAndRaise(1==getNbDim(),
		    "IloMap::get/set/at the map is not uni-dimensional.");
    IloInt i = _impl->getAbsoluteIndex(this->getEnv().makeSymbol(index));
    if (!(i>=0 && i<_impl->getSize()))
      throw IloMapOutOfBoundException(_impl, IloMapIndex(index));
    return *data(i);
  }

  Elt& at(IloTuple index) const {
    IloAssert (_impl, "Empty Handle in IloMap::get/set/at");
    IloTestAndRaise(1==getNbDim(),
		    "IloMap::get/set/at the map is not uni-dimensional.");
    IloInt i = _impl->getAbsoluteIndex(index);
    if (!(i>=0 && i<_impl->getSize()))
      throw IloMapOutOfBoundException(_impl, IloMapIndex(index));
    return *data(i);
  }

  Elt& at(IloSymbol index) const {
    IloAssert (_impl, "Empty Handle in IloMap::get/set/at");
    IloTestAndRaise(1==getNbDim(),
		    "IloMap::get/set/at the map is not uni-dimensional.");
    IloInt i = _impl->getAbsoluteIndex(index);
    if (!(i>=0 && i<_impl->getSize()))
      throw IloMapOutOfBoundException(_impl, IloMapIndex(index));
    return *data(i);
  }

  static IloSubMapI CreateData(IloEnvI* env,
			       IloBool validIdx,
			       IloInt currentDim,
			       IloMapIndexerI indexer) {
    IloInt rootNbDim = indexer.getSize();
    IloDiscreteDataCollectionI* currIdx = indexer[currentDim];
    IloInt size = currIdx->getSize();
    if (size <= 0) validIdx = IloFalse;
    if (!validIdx) size = 1;
    if (currentDim < rootNbDim-1) {
      IloArray<IloSubMapI> a(env, size);
      for (IloInt i=0; i<size; i++) {
	IloSubMapI elt = CreateData(env, validIdx, currentDim+1, indexer);
	a[i]= elt;
      }
      IloMapI* impl = new (env) IloMapI(env, rootNbDim-currentDim,
					currIdx, a.getImpl(), validIdx) ;
      impl->setType(IloMapItem::SubMap);
      return impl;
    } else {
      IloMap<Elt> a(env, currIdx, validIdx);
      return a.getImpl();
    }
  }

  static IloArrayI* GetSubArray(IloInt n, IloArrayI* array, IloInt index) {
    switch (n) {
    case 2: {
      IloArray< IloArray<Elt> >  a(array);
      return a[index].getImpl();
    }
    case 3: {
      IloArray< IloArray< IloArray<Elt> > >  a(array);
      return a[index].getImpl();
    }
    case 4: {
      IloArray< IloArray< IloArray< IloArray<Elt> > > >  a(array);
      return a[index].getImpl();
    }
    default:
      IloTestAndRaise(0, "this should not happen");
    }
    return 0;
  }
  static IloSubMapI CreateData(IloEnvI* env,
			       IloBool validIdx,
			       IloInt currentDim,
			       IloMapIndexerI indexer,
			       IloArrayI* values) {
    IloInt rootNbDim = indexer.getSize();
    IloInt nbDim = rootNbDim-currentDim;
    IloDiscreteDataCollectionI* currIdx = indexer[currentDim];
    IloInt size = currIdx->getSize();
    if (size <= 0) validIdx = IloFalse;
    if (!validIdx) size = 1;
    IloMapI* impl;
    if (nbDim > 1) {
      IloArray<IloSubMapI> a(env, size);
      for (IloInt i=0; i<size; i++) {
	IloSubMapI elt;
	if ((0==values) || (!validIdx))
	  elt = CreateData(env, validIdx, currentDim+1, indexer, 0);
	else elt = CreateData(env, validIdx, currentDim+1, indexer,
			      GetSubArray(nbDim, values, i));
	a[i]= elt;
      }
      impl = new (env) IloMapI(env, nbDim, currIdx, a.getImpl(), validIdx) ;
      impl->setType(IloMapItem::SubMap);
      return impl;
    } else {
      if ((0==values) || (!validIdx)) {
	IloMap<Elt> a(env, currIdx, validIdx);
	return a.getImpl();
      } else {
	IloMap<Elt> a(env, currIdx, IloArray<Elt>(values), validIdx);
	return a.getImpl();
      }
    }
  }

public:
  IloMap(IloMapI* impl) : _impl(impl) {}
  IloMap() : _impl(0) {}
  IloMapI* getImpl() const { return _impl; }
  IloMap(IloEnv env, IloDiscreteDataCollection indexer, IloBool validIdx=IloTrue) {
    _impl = new (env) IloMapI(env.getImpl(), 1,
			      indexer.getImpl(), (IloInt)sizeof(Elt), validIdx) ;
    _impl->setType(IloMapItem::GetType((Elt)0));
    //IloInt size = indexer.getSize();
    IloInt size = _impl->getNbElt();
    for (IloInt i = 0; i<size; i++)
      setData(i, Elt());
  }
  IloMap(IloEnv env, IloMapIndexer indexer) {
    IloInt dim = indexer.getSize();
    IloMapIndexerI indexerI(env, dim);
    for (IloInt i=0; i<dim; i++) indexerI[i] = indexer[i].getImpl();
    _impl = CreateData(env.getImpl(), IloTrue, 0, indexerI);
    indexerI.end();
  }

  IloMap(IloEnv env, IloMapIndexer indexer, Elt value) {
    IloInt dim = indexer.getSize();
    IloMapIndexerI indexerI(env, dim);
    for (IloInt i=0; i<dim; i++) indexerI[i] = indexer[i].getImpl();
    _impl = CreateData(env.getImpl(), IloTrue, 0, indexerI);
    indexerI.end();
	init(value);
  }
  operator IloArray<Elt>() const {
    IloAssert(_impl, "Empty Handle in IloMap::operator IloArray");
    IloTestAndRaise(1==getNbDim(),
		    "Cannot cast a multi dimensional map into an IloArray");
    return IloArray<Elt>(_impl->getValues());
  }

  void end() {
    IloAssert(_impl, "Empty Handle in IloMap::end");
    delete _impl;
    _impl = 0;
  }

  IloEnv getEnv() const {
    IloAssert(_impl, "Empty Handle in IloMap::getEnv");
    return _impl->getEnv();
  }

  IloInt getNbDim() const {
    IloAssert(_impl, "Empty Handle in IloMap::getNbDim");
    return _impl->getNbDim();
  }
  IloInt getSize() const {
    IloAssert(_impl, "Empty Handle in IloMap::getSize");
    return _impl->getSize();
  }
  IloInt getTotalSize() const {
    IloAssert(_impl, "Empty Handle in IloMap::getTotalSize");
    return _impl->getTotalSize();
  }
  IloDiscreteDataCollection getIndexer() const {
    IloAssert(_impl, "Empty Handle in IloMap::getIndexer");
    return IloDiscreteDataCollection(_impl->getIndexer());
  }

  IloDiscreteDataCollection getIndexer(IloInt i) const {
    IloAssert(_impl, "Empty Handle in IloMap::getIndexer");
    return IloDiscreteDataCollection(_impl->getIndexer(i));
  }

  IloMapIndexer makeMapIndexer() const {
    IloMapIndexer indexer(getEnv(), getNbDim());
    for (IloInt i=0; i<getNbDim(); i++) {
      indexer[i] = getIndexer(i);
    }
    return indexer;
  }

  IloSubMapI fastGetAtAbsoluteIndex(IloInt i) const {
    return *subArray(i);
  }

  IloSubMapI getAtAbsoluteIndex(IloInt i) const {
    IloAssert (_impl, "Empty Handle in IloMap::getAtAbsoluteIndex");
    IloTestAndRaise(getNbDim()>1, "Wrong dimension");
    IloInt size = _impl->getValueSize();
    if (!(i>=0 && i<size))
      throw IloMapOutOfBoundException(_impl, IloMapIndex(i));
    IloSubMapI sub = *subArray(i);
    return sub;
  }

  IloMap<Elt> operator[](IloInt i) const {
    IloAssert (_impl, "Empty Handle in IloMap::operator[]");
    i = _impl->getAbsoluteIndex(i);
    if (!(i>=0 && i<getSize()))
      throw IloMapOutOfBoundException(_impl, IloMapIndex(i));
    IloMap<Elt> sub(this->getAtAbsoluteIndex(i));
    return sub;
  }

  IloMap<Elt> getSub(IloInt i) const {
    return this->operator[](i);
  }

  IloMap<Elt> getSub(IloNum n) const {
    IloAssert (_impl, "Empty Handle in IloMap::set");
    IloInt i = _impl->getAbsoluteIndex(n);
    IloMap<Elt> sub(this->getAtAbsoluteIndex(i));
    return sub;
  }

  IloMap<Elt> getSub(const char* a) const {
    IloAssert (_impl, "Empty Handle in IloMap::set");
    IloInt i = _impl->getAbsoluteIndex(this->getEnv().makeSymbol(a));
    IloMap<Elt> sub(this->getAtAbsoluteIndex(i));
    return sub;
  }

  IloMap<Elt> operator[](IloNum n) const {
    IloAssert (_impl, "Empty Handle in IloMap::operator[]");
    IloInt i = _impl->getAbsoluteIndex(n);
    IloMap<Elt> sub(this->getAtAbsoluteIndex(i));
    return sub;
  }
  IloMap<Elt> operator[](const char* a) const {
    IloAssert (_impl, "Empty Handle in IloMap::operator[]");
    IloInt i = _impl->getAbsoluteIndex(this->getEnv().makeSymbol(a));
    IloMap<Elt> sub(this->getAtAbsoluteIndex(i));
    return sub;
  }
  IloMap<Elt> operator[](IloSymbol s) const {
    IloAssert (_impl, "Empty Handle in IloMap::operator[]");
    IloInt i = _impl->getAbsoluteIndex(s);
    IloMap<Elt> sub(this->getAtAbsoluteIndex(i));
    return sub;
  }

  IloMap<Elt> getSub(IloTuple i) const {
    return this->operator[](i);
  }

  IloMap<Elt> operator[](IloTuple t) const {
    IloAssert (_impl, "Empty Handle in IloMap::operator[]");
    IloInt i = _impl->getAbsoluteIndex(t);
    IloMap<Elt> sub(this->getAtAbsoluteIndex(i));
    return sub;
  }

  const Elt get(IloInt idx) const{
    return this->at(idx);
  }
  const Elt get(IloNum idx) const{
    return this->at(idx);
  }
  const Elt get(const char* idx) const{
    return this->at(this->getEnv().makeSymbol(idx));
  }
  const Elt get(IloTuple idx) const{
    return this->at(idx);
  }
  const Elt get(IloSymbol idx) const{
    return this->at(idx);
  }

  void set(IloInt idx, Elt value) {
    if (isLocked())
      throw IloLockedMapException(getImpl());
    this->at(idx) = value;
  }
  void set(IloNum idx, Elt value) {
    if (isLocked())
      throw IloLockedMapException(getImpl());
    this->at(idx) = value;
  }
  void set(const char* idx, Elt value) {
    if (isLocked())
      throw IloLockedMapException(getImpl());
    this->at(this->getEnv().makeSymbol(idx)) = value;
  }
  void set(IloTuple idx, Elt value) {
    if (isLocked())
      throw IloLockedMapException(getImpl());
    this->at(idx) = value;
  }
  void set(IloSymbol idx, Elt value) {
    if (isLocked())
      throw IloLockedMapException(getImpl());
    this->at(idx) = value;
  }

  Elt& fastEltAtAbsoluteIndex(IloInt i) const {
    return *data(i);
  }

  Elt& eltAtAbsoluteIndex(IloInt i) const {
    IloAssert(_impl, "Empty Handle in IloMap::eltAtAbsoluteIndex");
    IloTestAndRaise(1==getNbDim(),
		    "IloMap::eltAtAbsoluteIndex the map is not uni-dimensional.");
    //if (!(i>=0 && i<_impl->getSize()))
    if (!(i>=0 && i<_impl->getValueSize()))
      throw IloMapOutOfBoundException(_impl, IloMapIndex(i));
    return *data(i);
  }

  void setAtAbsoluteIndex(IloIntArray indices, IloMapItem value) {
    IloAssert (_impl, "Empty Handle in IloMap::setAtAbsoluteIndex");
    IloAssert (getNbDim() == indices.getSize(),
	       "IloMap::setAtAbsoluteIndex number of dimensions mismatch");
    _impl->setAtAbsoluteIndex(indices, value);
  }
  IloMapItem getAtAbsoluteIndex(IloIntArray indices) const {
    IloAssert (_impl, "Empty Handle in IloMap::getAtAbsoluteIndex");
    IloAssert (getNbDim() == indices.getSize(),
	       "IloMap::getAtAbsoluteIndex number of dimensions mismatch");
    return _impl->getAtAbsoluteIndex(indices);
  }

  void setAt(IloMapIndexArray indices, Elt value) {
    IloAssert (_impl, "Empty Handle in IloMap::setAt");
    _impl->setAt(indices, IloMapItem(value));
  }
  Elt getAt(IloMapIndexArray indices) const {
    IloAssert (_impl, "Empty Handle in IloMap::getAt");
    IloAssert (getNbDim() == indices.getSize(),
	       "IloMap::getAt number of dimensions mismatch");
    IloAssert(indices.isConstant(),
	      "IloMap::getAt does not accept non constant indices.");
    IloMapItem item = _impl->getAt(indices);
    Elt res;
    item.getValue(res);
    return res;
  }
  IloMapI* copy() const {
    IloAssert (_impl, "Empty Handle in IloMap::copy");
    return _impl->getCopy();
  }
  IloMapI* lock() const {
    if (_impl != 0) _impl->lock();
    return _impl;
  }
  IloMapI* unlock() const {
    if (_impl != 0) _impl->unlock();
    return _impl;
  }
  IloBool isLocked() const {
    IloAssert (_impl, "Empty Handle in IloMap::isLocked");
    return _impl->isLocked();
  }

  const char * getName() const {
    if (getImpl() == 0)
      throw IloEmptyHandleException("IloMap::getName : Using empty handle");
    return getImpl()->getName();
  }
  void setName(const char * name) const {
    if (getImpl() == 0)
      throw IloEmptyHandleException("IloMap::setName : Using empty handle");
    getImpl()->setName(name);
  }

  class Iterator : public IloMapIterator {
  public:
    Iterator(const IloMapI* m = 0)
      : IloMapIterator(m->getEnv()->getGeneralAllocator(), m) {
    }
    Iterator(const IloMap m)
      : IloMapIterator(m.getImpl()->getEnv()->getGeneralAllocator(),
		       m.getImpl()) {
    }

    Iterator(IloGenAlloc* heap, const IloMapI* m): IloMapIterator(heap, m) {
    }

    virtual ~Iterator(){}

    Elt& operator*() {
      IloArray<Elt> a(this->getCurrentEltArray());
      return a[this->getCurrentDeepestIndex()];
    }
  };
  Iterator* iterator(IloGenAlloc* heap) const {
    IloAssert(getImpl() != 0, "IloMap: Using empty handle");
    return new (getEnv()) Iterator(heap, _impl);

  }
  Iterator* iterator() const {
    IloAssert(getImpl() != 0, "IloMap: Using empty handle");
    return new (getEnv()) Iterator(_impl);

  }
  IloArray<Elt> makeArray() const {
    IloAssert(_impl, "Empty Handle in IloMap::makeArray");
    IloArray<Elt> array(getEnv(), getImpl()->getTotalSize());
	Iterator* it = iterator(getEnv().getGeneralAllocator());
	for(IloInt i=0; it->ok(); ++(*it)) {
		array[i++] = **it;
	}
	delete it;
    return array;
  }


  void init(Elt elt){
#if defined(ILO_HP_aCC) || defined(ILO_LINUX)
    for (typename IloMap::Iterator it(this->getImpl()); it.ok(); ++it) {
#else
    for (IloMap::Iterator it(this->getImpl()); it.ok(); ++it) {
#endif
      *it = elt;
    }
  }
};


template <class X>
inline ILOSTD(ostream)& operator<<(ILOSTD(ostream)& out, const IloMap<X>& m) {
  if (m.getImpl()) {
    IloInt nb = m.getNbDim();
    IloBool oneLine = (m.getTotalSize()<=15);
    if (nb>1) {
      IloMap<X> ma;
      out << '[';
      IloInt i, n=m.getSize()-1;
      for (i = 0; i < n; ++i) {
	ma = IloMap<X>(m.getAtAbsoluteIndex(i));
	out << ma << ',';
	if (oneLine) out << ' ';
	else         out << ILOSTD(endl);
      }
      if (n>=0) {
	ma = IloMap<X>(m.getAtAbsoluteIndex(i));
	out << ma;
      }
      out << ']';
      if (!oneLine)
	out << ILOSTD(endl);
    } else if (1==nb) {
      out << '[';
      IloInt i, n=m.getSize()-1;
      for (i = 0; i < n; ++i) {
	out << m.eltAtAbsoluteIndex(i);
	out << ',';
	if ( (i+1) % 10 ) out << ' ';
	else              out << ILOSTD(endl);
      }
      if (n>=0)
	out << m.eltAtAbsoluteIndex(n);
      out << ']';
    } else
      out << "[]";
  }
  return out;
}

#ifdef CPPREF_GENERATION
#define name2(x,y) x##y
#endif

#define ILOMAPCTORBASE(_typeMap, _base)\
public:\
  typedef _base super;\
  _typeMap(const _typeMap & cpy) : _base(cpy) {}\
  _typeMap(IloMapI* impl) : _base(impl) {}\
  _typeMap() : _base() {}\
private:

#define ILOMAPCTORDECL(_type,_base)\
  ILOMAPCTORBASE(name2(_type,Map),_base)\
public:\
  name2(_type,Map)(IloEnv env, IloMapIndexer indexer);\
  name2(_type,Map)(IloEnv env, IloMapIndexer indexer, _type elt);\
  name2(_type,Map)(IloEnv env, IloDiscreteDataCollection indexer);\
private:

#define ILOMAPCTORIMPL_ALLTYPES(_type, _typeMap,_base, _item)\
  _typeMap::_typeMap(IloEnv env, IloMapIndexer indexer)\
                    : _base(env, indexer) {\
     getImpl()->setType(_item);\
  }\
  _typeMap::_typeMap(IloEnv env, IloMapIndexer indexer, _type elt)\
                    : _base(env, indexer, elt) {\
     getImpl()->setType(_item);\
  }\
  _typeMap::_typeMap(IloEnv env,\
		       IloDiscreteDataCollection indexer)\
                    : _base(env, indexer) {\
     getImpl()->setType(_item);\
  }


#define ILOMAPCTORIMPL(_type,_base,_item)\
ILOMAPCTORIMPL_ALLTYPES(_type, name2(_type,Map),_base,_item)

/////////////////////////////////////
//
// IloMap<base type>
//

typedef IloMap<IloInt> IloIntMapBase;
class IloIntMap : public IloIntMapBase {
public:
  ILOMAPCTORDECL(IloInt, IloIntMapBase)
public:
  REDEFINE_SIMPLE_ACCESSORS_BOUND(IloInt, IloIntMap)
  DEFINE_EXPR_ACCESSORS(IloIntSubMapExpr)
  operator IloIntArray() {
    IloArray<IloInt> a = this->IloIntMapBase::operator IloArray<IloInt>();
    return IloIntArray(a.getImpl());
  }
  #ifdef CPPREF_GENERATION
  void set(const char* index, IloInt value);
   void set(IloInt index, IloInt value);
   void set(IloNum index, IloInt value);
  void set(IloTuple index, IloInt value);
  void set(IloSymbol index, IloInt value);
  #endif
};

typedef IloMap<IloNum> IloNumMapBase;
class IloNumMap : public IloNumMapBase {
public:
  ILOMAPCTORDECL(IloNum, IloNumMapBase)
public:
  REDEFINE_SIMPLE_ACCESSORS_BOUND(IloNum, IloNumMap)
  DEFINE_EXPR_ACCESSORS(IloNumSubMapExpr)
  operator IloNumArray() {
    IloArray<IloNum> a = this->IloNumMapBase::operator IloArray<IloNum>();
    return IloNumArray(a.getImpl());
  }
  #ifdef CPPREF_GENERATION
  void set(const char* index, IloNum value);
   void set(IloInt index, IloNum value);
   void set(IloNum index, IloNum value);
  void set(IloTuple index, IloNum value);
  void set(IloSymbol index, IloNum value);
  #endif
};

typedef IloMap<IloSymbol> IloSymbolMapBase;
class IloSymbolMap : public IloSymbolMapBase {
public:
  ILOMAPCTORDECL(IloSymbol, IloSymbolMapBase)
public:
  REDEFINE_ACCESSORS_BOUND(IloSymbol, IloSymbolMap)
  DEFINE_EXPR_ACCESSORS(IloSymbolSubMapExpr)
  operator IloSymbolArray() {
    IloArray<IloSymbol> a = this->IloSymbolMapBase::operator IloArray<IloSymbol>();
    return IloSymbolArray(a.getImpl());
  }
  using IloSymbolMapBase::set;
  void set(IloInt index, const char* value) {
    set(index, getEnv().makeSymbol(value));
  }
  //void set(const int idx, const char* value) { set((IloInt)idx, value); }
  //void set(const char idx, const char* value) { set((IloInt)idx, value); }
  //void set(const short idx, const char* value) { set((IloInt)idx, value); }
  void set(IloNum index, const char* value) {
    set(index, getEnv().makeSymbol(value));
  }
  void set(const char* index, const char* value) {
    set(getEnv().makeSymbol(index), getEnv().makeSymbol(value));
  }
  void set(IloTuple index, const char* value) {
    set(index, getEnv().makeSymbol(value));
  }
  void set(IloSymbol idx, const char* value) {
    set(idx, getEnv().makeSymbol(value));
  }
};

typedef IloMap<IloAnyCollection> IloAnyCollectionMapBase;
class IloAnyCollectionMap : public IloAnyCollectionMapBase {
public:
  ILOMAPCTORDECL(IloAnyCollection, IloAnyCollectionMapBase)
public:
  REDEFINE_ACCESSORS_BOUND(IloAnyCollection, IloAnyCollectionMap)
  DEFINE_EXPR_ACCESSORS(IloAnyCollectionSubMapExpr)
  void endElements();
  void setAtAbsoluteIndex(IloIntArray indices, IloAnyCollection value);
  void setAtAbsoluteIndex(IloInt idx, IloAnyCollection value);
  void setAt(IloMapIndexArray indices, IloAnyCollection value);
};

class IloSymbolSetMap : public IloAnyCollectionMap {
public:
  ILOMAPCTORDECL(IloSymbolSet, IloAnyCollectionMap)
public:
  REDEFINE_ACCESSORS_BOUND(IloSymbolSet, IloSymbolSetMap)
  REDEFINE_EXPR_ACCESSORS_CASTFROMIMPL(IloSymbolCollectionSubMapExpr)

  #ifdef CPPREF_GENERATION
  void set(const char* index, IloSymbolSet value);
   void set(IloInt index, IloSymbolSet value);
   void set(IloNum index, IloSymbolSet value);
  void set(IloTuple index, IloSymbolSet value);
  void set(IloSymbol index, IloSymbolSet value);
  #endif
  void initCollections();
};

class IloTupleSetMap : public IloAnyCollectionMap {
public:
  ILOMAPCTORBASE(IloTupleSetMap,IloAnyCollectionMap)
public:
  IloTupleSetMap(IloEnv env, IloTupleSchema schema, IloMapIndexer indexer);
public:
  REDEFINE_ACCESSORS_BOUND(IloTupleSet, IloTupleSetMap)
  REDEFINE_EXPR_ACCESSORS_CASTFROMIMPL(IloTupleSetSubMapExpr)
  IloTupleSchema getSchema() const {
    IloAssert(getImpl(), "IloTupleSetMap::getSchema using empty handle");
    return (IloTupleSchemaI*)getImpl()->getAuxData();
  }
  #ifdef CPPREF_GENERATION
  void set(const char* index, IloTupleSet value);
   void set(IloInt index, IloTupleSet value);
   void set(IloNum index, IloTupleSet value);
  void set(IloTuple index, IloTupleSet value);
  void set(IloSymbol index, IloTupleSet value);
  #endif
  void initCollections();
};

typedef IloMap<IloIntCollection> IloIntCollectionMapBase;
class IloIntCollectionMap : public IloIntCollectionMapBase {
public:
  ILOMAPCTORDECL(IloIntCollection, IloIntCollectionMapBase)
public:
  REDEFINE_ACCESSORS_BOUND(IloIntCollection, IloIntCollectionMap)
  DEFINE_EXPR_ACCESSORS(IloIntCollectionSubMapExpr)
  void endElements();
  void setAtAbsoluteIndex(IloIntArray indices, IloIntCollection value);
  void setAtAbsoluteIndex(IloInt idx, IloIntCollection value);
  void setAt(IloMapIndexArray indices, IloIntCollection value);
};

class IloIntSetMap : public IloIntCollectionMap {
public:
  ILOMAPCTORDECL(IloIntSet, IloIntCollectionMap)
public:
  REDEFINE_ACCESSORS_BOUND(IloIntSet, IloIntSetMap)
  REDEFINE_EXPR_ACCESSORS_CASTFROMIMPL(IloIntCollectionSubMapExpr)
  #ifdef CPPREF_GENERATION
  void set(const char* index, IloIntSet value);
   void set(IloInt index, IloIntSet value);
   void set(IloNum index, IloIntSet value);
  void set(IloTuple index, IloIntSet value);
  void set(IloSymbol index, IloIntSet value);
  #endif
  void initCollections();
};

typedef IloMap<IloNumCollection> IloNumCollectionMapBase;
class IloNumCollectionMap : public IloNumCollectionMapBase {
public:
  ILOMAPCTORDECL(IloNumCollection, IloNumCollectionMapBase)
public:
  REDEFINE_ACCESSORS_BOUND(IloNumCollection, IloNumCollectionMap)
  DEFINE_EXPR_ACCESSORS(IloNumCollectionSubMapExpr)
  void endElements();
  void setAtAbsoluteIndex(IloIntArray indices, IloNumCollection value);
  void setAtAbsoluteIndex(IloInt idx, IloNumCollection value);
  void setAt(IloMapIndexArray indices, IloNumCollection value);
};

class IloNumSetMap : public IloNumCollectionMap {
public:
  ILOMAPCTORDECL(IloNumSet, IloNumCollectionMap)
public:
  REDEFINE_ACCESSORS_BOUND(IloNumSet, IloNumSetMap)
  REDEFINE_EXPR_ACCESSORS_CASTFROMIMPL(IloNumCollectionSubMapExpr)
  #ifdef CPPREF_GENERATION
  void set(const char* index, IloNumSet value);
   void set(IloInt index, IloNumSet value);
   void set(IloNum index, IloNumSet value);
  void set(IloTuple index, IloNumSet value);
  void set(IloSymbol index, IloNumSet value);
  #endif
  void initCollections();
};


/////////////////////////////////////
//
// IloMap<extractable>
//

#define REDEFINEFROMSUPER_ALLTYPES(_eltType, _eltMap, _eltArray, _superMap)\
  ILOMAPCTORDECL(_eltType, _superMap)\
public:\
  REDEFINE_ACCESSORS_BOUND(_eltType, _eltMap)\
  _eltMap getClone(IloEnvI* env) const {\
    return _eltMap((_superMap::getClone(env)).getImpl());\
  }\
  operator _eltArray() {\
    IloArray<IloExtractable> a = this->IloExtractableMapBase::operator IloArray<IloExtractable>();\
    return _eltArray(a.getImpl());\
  }

#define REDEFINEFROMSUPER(_eltType, _superEltType)\
  REDEFINEFROMSUPER_ALLTYPES(_eltType,\
                             name2(_eltType, Map),\
                             name2(_eltType, Array),\
                             name2(_superEltType, Map))

typedef IloMap<IloExtractable> IloExtractableMapBase;
class IloExtractableMap : public IloExtractableMapBase {
private:
  IloSubMapI getClone(IloEnvI* env, IloInt dim, IloSubMapI srcI) const;
public:
  ILOMAPCTORDECL(IloExtractable, IloExtractableMapBase)
public:
  REDEFINE_ACCESSORS_BOUND(IloExtractable, IloExtractableMap)
  IloExtractableMap getClone(IloEnvI*) const;
  void replaceByClone(IloEnvI* env);
  operator IloExtractableArray() {
    IloArray<IloExtractable> a = this->IloExtractableMapBase::operator IloArray<IloExtractable>();
    return IloExtractableArray(a.getImpl());
  }
  IloExtractableArray asNewExtractableArray() const;
  void endElements();
};

typedef IloMap<IloConstraint>::Iterator IloConstraintMapIterator;
class IloConstraintMap : public IloExtractableMap {
private:
  void release(IloMapI* m);
public:
// seems to be overlapping the REDEFINEFROMSUPER below ...
// does it break cpp doc if I comment it?
//#ifdef CPPREF_GENERATION
//  REDEFINE_ACCESSORS_BOUND(IloConstraint, IloConstraintMap)
//#endif
// RK: Needed to define REDEFINEFROMSUPER in config file. Okay now.
  REDEFINEFROMSUPER(IloConstraint, IloExtractable)
  void release() {
    IloAssert(getImpl(), "Empty Handle in IloConstraint::release");
    release(getImpl());
  }
};

typedef IloMap<IloIntExpr>::Iterator IloIntExprMapIterator;
class IloIntExprMap : public IloExtractableMap {
private:
  void release(IloMapI* m);
  void lockExprs(IloMapI* m);
public:
  REDEFINEFROMSUPER(IloIntExpr, IloExtractable)
  DEFINE_EXPR_ACCESSORS(IloIntExprSubMapExpr)
  void release() {
    IloAssert(getImpl(), "Empty Handle in IloIntExprMap::release");
    release(getImpl());
  }
  void lockExprs() {
    IloAssert(getImpl(), "Empty Handle in IloIntExprMap::lockExprs");
    lockExprs(getImpl());
  }
};

typedef IloMap<IloIntVar>::Iterator IloIntVarMapIterator;
class IloIntVarMap : public IloIntExprMap {
public:
  REDEFINEFROMSUPER(IloIntVar, IloIntExpr)
  REDEFINE_EXPR_ACCESSORS(IloIntExprSubMapExpr)
  IloIntExprMap toIntExprMap() const;
  void initVars(IloInt lb, IloInt ub);
  #ifdef CPPREF_GENERATION
  void set(const char* index, IloIntVar value);
   void set(IloInt index, IloIntVar value);
   void set(IloNum index, IloIntVar value);
  void set(IloTuple index, IloIntVar value);
  void set(IloSymbol index, IloIntVar value);
  #endif
};

typedef IloMap<IloBoolVar>::Iterator IloBoolVarMapIterator;
class IloBoolVarMap : public IloIntVarMap {
public:
  ILOMAPCTORDECL(IloBoolVar, IloIntVarMap)
public:
  IloBoolVarMap getClone(IloEnvI* env) const {
    return IloBoolVarMap((IloIntVarMap::getClone(env)).getImpl());
  }
  operator IloBoolVarArray() {
    IloArray<IloExtractable> a = this->IloExtractableMapBase::operator IloArray<IloExtractable>();
    return IloBoolVarArray(a.getImpl());
  }
  REDEFINE_ACCESSORS_BOUND(IloBoolVar, IloBoolVarMap)
  REDEFINE_EXPR_ACCESSORS(IloIntExprSubMapExpr)
};

typedef IloMap<IloNumExpr>::Iterator IloNumExprMapIterator;
class IloNumExprMap : public IloExtractableMap {
private:
  void release(IloMapI* m);
  void lockExprs(IloMapI* m);
public:
  REDEFINEFROMSUPER(IloNumExpr, IloExtractable)
  DEFINE_EXPR_ACCESSORS(IloNumExprSubMapExpr)
  void release() {
    IloAssert(getImpl(), "Empty Handle in IloNumExprMap::release");
    release(getImpl());
  }
  void lockExprs() {
    IloAssert(getImpl(), "Empty Handle in IloNumExprMap::lockExprs");
    lockExprs(getImpl());
  }
};

typedef IloMap<IloNumVar>::Iterator IloNumVarMapIterator;
class IloNumVarMap : public IloNumExprMap {
public:
  REDEFINEFROMSUPER(IloNumVar, IloNumExpr)
  REDEFINE_EXPR_ACCESSORS(IloNumExprSubMapExpr)
  IloNumExprMap toNumExprMap() const;
  void initVars(IloNum lb, IloNum ub);
  #ifdef CPPREF_GENERATION
  void set(const char* index, IloNumVar value);
   void set(IloInt index, IloNumVar value);
   void set(IloNum index, IloNumVar value);
  void set(IloTuple index, IloNumVar value);
  void set(IloSymbol index, IloNumVar value);
  #endif
};


// OPL dexpr stuff


class IloIntExprMapLightI : public IloAbstractMapI {
  ILORTTIDECL
  IloIntExprI* _expr;
  IloMapIndexer _indexer;
  IloMapIndexArray _indexes;
  char* _name;
public:
  IloIntExprMapLightI(IloEnvI* env, IloIntExprArg expr,
                      IloMapIndexer indexer, IloMapIndexArray indexes);
  virtual ~IloIntExprMapLightI();
  virtual IloDiscreteDataCollectionI* getIndexer() const;
  virtual IloDiscreteDataCollectionI* getIndexer(IloInt i) const;
  virtual void lock();
  virtual void unlock();
  virtual IloBool isLocked() const;
  virtual const char* getName() const { return _name; }
  virtual void setName(const char* name);
  virtual IloInt getNbDim() const;
  virtual IloInt getTotalSize() const;
  virtual IloInt getSize() const;
  virtual IloMapItem getAt(IloMapIndexArray indices) const;
  virtual void setAt(IloMapIndexArray indices, IloMapItem value);
  virtual void setAtAbsoluteIndex(IloIntArray indices, IloMapItem value);
  virtual IloMapItem getAtAbsoluteIndex(IloIntArray indices) const;
  virtual void display(ILOSTD(ostream)& out) const;
  IloMapIndexArray getSharedIndex(IloIntArray indices);
};


class IloIntDExprI : public IloIntExprI {
  ILOEXTRDECL
  IloIntExprI* _expr;
  IloMapIndexArray _indexes;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloIntDExprI(IloEnvI* env, IloIntExprI* expr, const char* name=0);
  IloIntDExprI(IloEnvI* env, IloIntExprI* expr, IloMapIndexArray indexes, const char* name=0);
  virtual ~IloIntDExprI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloIntExprI* getExpr() const { return _expr; }
  IloMapIndexArray getIndexes() const {return _indexes;}
};

class IloIntDExprMap {
  IloIntExprMapLightI* _impl;
public:
    IloIntDExprMap(IloIntExprMapLightI* impl):_impl(impl) {
    }
  IloIntDExprMap(IloEnv env, IloIntExprArg expr,
                      IloMapIndexer indexer, IloMapIndexArray indexes);
  IloIntExprMapLightI* getImpl() const {
    return _impl;
  }
  void end() {
    delete _impl;
    _impl = 0;
  }
  IloIntExprArg get(IloTuple index) const {
    IloAssert(_impl, "Using empty IloIntDExprMap handle");
    IloMapIndexArray a(_impl->getEnv(), 1, index);
    IloIntExprArg e = getAt(a);
    a.end();
    return e;
  }
  IloIntExprArg get(const char* index) const {
    IloAssert(_impl, "Using empty IloIntDExprMap handle");
    IloMapIndexArray a(_impl->getEnv(), 1, index);
    IloIntExprArg e = getAt(a);
    a.end();
    return e;
  }
  IloIntExprArg get(IloSymbol index) const {
    IloAssert(_impl, "Using empty IloIntDExprMap handle");
    IloMapIndexArray a(_impl->getEnv(), 1, index);
    IloIntExprArg e = getAt(a);
    a.end();
    return e;
  }
  IloIntExprArg get(IloNum index) const {
    IloAssert(_impl, "Using empty IloIntDExprMap handle");
    IloMapIndexArray a(_impl->getEnv(), 1, index);
    IloIntExprArg e = getAt(a);
    a.end();
    return e;
  }
  IloIntExprArg get(IloInt index) const {
    IloAssert(_impl, "Using empty IloIntDExprMap handle");
    IloMapIndexArray a(_impl->getEnv(), 1, index);
    IloIntExprArg e = getAt(a);
    a.end();
    return e;
  }
  IloIntExprArg getAt(IloMapIndexArray indices) const {
    IloAssert(_impl, "Using empty IloIntDExprMap handle");
    return _impl->getAt(indices).asIntExpr();
  }
  IloIntExprArg operator[](IloTuple index) const {
    return get(index);
  }
  IloIntExprArg operator[](const char* index) const {
    return get(index);
  }
  IloIntExprArg operator[](IloSymbol index) const {
    return get(index);
  }
  IloIntExprArg operator[](IloNum index) const {
    return get(index);
  }
  IloIntExprArg operator[](IloInt index) const {
    return get(index);
  }
};




class IloIntMapLightIterator {
protected:
  const IloIntExprMapLightI* _map;
  IloIntArray _size;
  IloIntArray _index;
  IloBool _ok;
  virtual IloBool next(IloInt);
  void resetIndex(IloInt i);
  void checkVectorIndex();
public:
	IloEnvI* getEnv() const{
		return _map->getEnv();
	}
  virtual IloBool next();
  virtual void reset();
  IloIntMapLightIterator(IloIntDExprMap m);
  IloIntMapLightIterator(const IloIntExprMapLightI* m);
  virtual ~IloIntMapLightIterator();
  const IloIntExprMapLightI* getIntMapI() const { return _map; }
  IloMapIndexArray getSharedIndex(IloIntArray indices){
	  return ((IloIntExprMapLightI*)getIntMapI())->getSharedIndex(indices);
  }

  IloBool ok() const {
	  return _ok;
  }
  void setNotOk() { _ok = IloFalse; }

  void operator++() {
	  _ok = next();
  }

  void operator delete(void *p, size_t sz);
#ifdef ILODELETEOPERATOR
  void operator delete(void *p, const IloEnvI *);
  void operator delete(void *p, const IloEnv &);
#endif

  IloIntDExprI* operator*() {
	  IloMapItem item = _map->getAtAbsoluteIndex(_index);
	  return (IloIntDExprI*)item.asIntExpr();
  }
};


class IloNumExprMapLightI : public IloAbstractMapI {
  ILORTTIDECL
  IloNumExprI* _expr;
  IloMapIndexer _indexer;
  IloMapIndexArray _indexes;
  char* _name;
public:
  IloNumExprMapLightI(IloEnvI* env, IloNumExprArg expr,
                      IloMapIndexer indexer, IloMapIndexArray indexes);
  virtual ~IloNumExprMapLightI();
  virtual IloDiscreteDataCollectionI* getIndexer() const;
  virtual IloDiscreteDataCollectionI* getIndexer(IloInt i) const;
  virtual void lock();
  virtual void unlock();
  virtual IloBool isLocked() const;
  virtual const char* getName() const { return _name; }
  virtual void setName(const char* name);
  virtual IloInt getNbDim() const;
  virtual IloInt getTotalSize() const;
  virtual IloInt getSize() const;
  virtual IloMapItem getAt(IloMapIndexArray indices) const;
  virtual void setAt(IloMapIndexArray indices, IloMapItem value);
  virtual void setAtAbsoluteIndex(IloIntArray indices, IloMapItem value);
  virtual IloMapItem getAtAbsoluteIndex(IloIntArray indices) const;
  virtual void display(ILOSTD(ostream)& out) const;
  IloMapIndexArray getSharedIndex(IloIntArray indices);
};

class IloNumDExprI : public IloNumExprI {
  ILOEXTRDECL
  IloNumExprI* _expr;
  IloMapIndexArray _indexes;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloNumDExprI(IloEnvI* env, IloNumExprI* expr, const char* name=0);
  IloNumDExprI(IloEnvI* env, IloNumExprI* expr, IloMapIndexArray indexes, const char* name=0);
  virtual ~IloNumDExprI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloNumExprI* getExpr() const { return _expr; }
  IloMapIndexArray getIndexes() const {return _indexes;}
};

class IloNumDExprMap {
  IloNumExprMapLightI* _impl;
public:
    IloNumDExprMap(IloNumExprMapLightI* impl):_impl(impl) {
    }
	IloNumDExprMap(IloEnv env, IloNumExprArg expr,
                      IloMapIndexer indexer, IloMapIndexArray indexes);

  IloNumExprMapLightI* getImpl() const {
    return _impl;
  }
  void end() {
    delete _impl;
    _impl = 0;
  }
  IloNumExprArg get(IloTuple index) const {
    IloAssert(_impl, "Using empty IloNumDExprMap handle");
    IloMapIndexArray a(_impl->getEnv(), 1, index);
    IloNumExprArg e = getAt(a);
    a.end();
    return e;
  }
  IloNumExprArg get(const char* index) const {
    IloAssert(_impl, "Using empty IloNumDExprMap handle");
    IloMapIndexArray a(_impl->getEnv(), 1, index);
    IloNumExprArg e = getAt(a);
    a.end();
    return e;
  }
  IloNumExprArg get(IloSymbol index) const {
    IloAssert(_impl, "Using empty IloNumDExprMap handle");
    IloMapIndexArray a(_impl->getEnv(), 1, index);
    IloNumExprArg e = getAt(a);
    a.end();
    return e;
  }
  IloNumExprArg get(IloNum index) const {
    IloAssert(_impl, "Using empty IloNumDExprMap handle");
    IloMapIndexArray a(_impl->getEnv(), 1, index);
    IloNumExprArg e = getAt(a);
    a.end();
    return e;
  }
  IloNumExprArg get(IloInt index) const {
    IloAssert(_impl, "Using empty IloNumDExprMap handle");
    IloMapIndexArray a(_impl->getEnv(), 1, index);
    IloNumExprArg e = getAt(a);
    a.end();
    return e;
  }
  IloNumExprArg getAt(IloMapIndexArray indices) const {
    IloAssert(_impl, "Using empty IloNumDExprMap handle");
    return _impl->getAt(indices).asNumExpr();
  }
  IloNumExprArg operator[](IloTuple index) const {
    return get(index);
  }
  IloNumExprArg operator[](const char* index) const {
    return get(index);
  }
  IloNumExprArg operator[](IloSymbol index) const {
    return get(index);
  }
  IloNumExprArg operator[](IloNum index) const {
    return get(index);
  }
  IloNumExprArg operator[](IloInt index) const {
    return get(index);
  }
};




class IloNumMapLightIterator {
protected:
  const IloNumExprMapLightI* _map;
  IloIntArray _size;
  IloIntArray _index;
  IloBool _ok;
  virtual IloBool next(IloInt);
  void resetIndex(IloInt i);
  void checkVectorIndex();
public:
	IloEnvI* getEnv() const{
		return _map->getEnv();
	}
  virtual IloBool next();
  virtual void reset();
  IloNumMapLightIterator(IloNumDExprMap m);
  IloNumMapLightIterator(const IloNumExprMapLightI* m);
  virtual ~IloNumMapLightIterator();
  const IloNumExprMapLightI* getNumMapI() const { return _map; }
  IloMapIndexArray getSharedIndex(IloIntArray indices){
	  return ((IloNumExprMapLightI*)getNumMapI())->getSharedIndex(indices);
  }

  IloBool ok() const {
    return _ok;
  }
  void setNotOk() { _ok = IloFalse; }

  void operator++() {
    _ok = next();
  }

  void operator delete(void *p, size_t sz);
#ifdef ILODELETEOPERATOR
  void operator delete(void *p, const IloEnvI *);
  void operator delete(void *p, const IloEnv &);
#endif

  IloNumDExprI* operator*() {
	  IloMapItem item = _map->getAtAbsoluteIndex(_index);
	  return (IloNumDExprI*)item.asNumExpr();
  }
};

#endif

