// ---------------------------------------------------*- C++ -*--------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_ilotuplecollectioniH
#define __CONCERT_ilotuplecollectioniH

#if !defined(__CONCERT_ilotuplecollectionH)
# include <ilconcert/ilotuplecollection.h>
#endif
#if !defined(__CONCERT_ilotupleH)
# include <ilconcert/ilotuple.h>
#endif

#if !defined(__CONCERT_ilocollectioncolumniH)
# include <ilconcert/iltuple/ilocollectioncolumni.h>
#endif

#if !defined(__CONCERT_ilosetH)
# include <ilconcert/iloset.h>
#endif

#if !defined(__CONCERT_ilotuplecollectionhashH)
# include <ilconcert/iltuple/ilotuplecollectionhash.h>
#endif

#if !defined(__CONCERT_ilotuplerequestH)
# include <ilconcert/iltuple/ilotuplerequest.h>
#endif

#if !defined(__CONCERT_ilotupleschemaiH)
#include <ilconcert/iltuple/ilotupleschemai.h>
#endif


class IloTupleSet;
class IloTupleIterator;
class IloTupleSetArray;
class IloTupleI;
class IloTuple;
class IloTupleBuffer;
class IloTupleBufferI;
class IloTupleSchema;
class IloIntArray;

class IloTuplePattern;
class IloTuplePatternI;

class IloIntCollectionColumnI;
class IloNumCollectionColumnI;
class IloAnyCollectionColumnI;
class IloExtendedComprehensionI;
class IloTupleSetArray;

class IloTupleException : public IloException {
public:
  IloTupleException(const char* m):IloException(m){}
};

class IloTupleDefinitionI : public IloColumnDefinitionI{
	ILORTTIDECL
private:
  IloTupleSchema _schema;
public:
  IloTupleDefinitionI(IloEnv env, IloTupleSchema schema, const char* name = 0);
  virtual IloBool isTuple() const;
  IloTupleSchema getSchema() const;
  virtual void display(ILOSTD(ostream)& out) const;
};


class IloAbstractTupleChangeListenerI : public IloDataChangeListenerI {
  ILORTTIDECL
  friend class IloTupleCollectionChangeNotifierI;
protected :
  virtual void change() const = 0;
  virtual void changeTuple(IloInt) const = 0;
  virtual void addTuple(IloInt) const = 0;
  virtual void removeTuple(IloInt) const = 0;
public:
  IloAbstractTupleChangeListenerI(IloEnvI* env) : IloDataChangeListenerI(env) {}
  virtual ~IloAbstractTupleChangeListenerI() {};
};

class IloTupleCollectionChangeNotifierI : public IloDataChangeNotifierI {
  ILORTTIDECL
  friend class IloTupleSetI;
  friend class IloTupleCollectionI;
  IloTupleSetI* _tupleSet;
protected:
  virtual void change() const;
  virtual void changeTuple(IloInt) const;
  virtual void addTuple(IloInt) const;
  virtual void removeTuple(IloInt) const;
public:
  IloTupleCollectionChangeNotifierI(IloEnvI*, IloTupleSetI*);
};


class IloTupleSetChangeListenerI : public IloAbstractTupleChangeListenerI {
  ILORTTIDECL
  IloTupleSetI* _set;
public:
  IloTupleSetChangeListenerI(IloEnvI* env, IloTupleSetI* request);
  virtual ~IloTupleSetChangeListenerI();
public:
  virtual void change() const;
  virtual void changeTuple(IloInt) const;
  virtual void addTuple(IloInt) const;
  virtual void removeTuple(IloInt) const;
private:
	IloEnvI* getEnv() const;
	IloTupleSetI* getSet() const;
};


class IloTupleCollectionI : public IloAnyCollectionI {
	ILORTTIDECL
protected:
	IloInt _size;
	IloIntArray _empty;
	IloDataCollectionArray _array;
	IloTupleSchema _schema;

    IloTupleBufferI* _sharedBuffer;
    IloTupleCellArray _sharedTupleCells;
	IloTupleCellArray _sharedKeyCells;
	IloTupleCellArray _sharedRefSliceCells;
	IloTupleCellArray _sharedRefCells;

	// for internal use to avoid conflicts with sharedxxxCells
	IloTupleCellArray _internalTupleCells;

	// for reference resolution
	IloTupleCellArray _referenceCells;

	IloTupleCells2ArrayIndexHash* _hashForTupleCollection;
	IloAnyArray _pointers;
	IloIntArray _collectionColumnIdx;
	IloBool _hasReference;
	IloIntArray _referencedColumns;
	IloIntArray _referenceKeyColumns;
	IloTupleCellArray getNormalizedCommitCells(IloTupleCellArray line);
	IloTupleCellArray getNormalizedFindCells(IloTupleCellArray line);
protected :
	IloTupleCollectionChangeNotifierI* _changeNotifier;
public:
	IloTupleCollectionChangeNotifierI* getChangeNotifier(IloTupleSetI* set);
	virtual IloTupleCollectionChangeNotifierI* getChangeNotifier();
	IloTupleCellArray makeDefaultValue() const;
	void freeNormalizedCells(IloTupleCellArray line);
	void checkReferences() const;

protected:
	void setListeners(IloTupleSetI*);
	void init(IloEnv env, IloTupleSchema schema, IloInt n = 0);
	void disableSelectIndexesOnColumn();
	void enableSelectIndexesOnColumn();
	IloBool isSelectIndexEnabledOnColumn() const;

	//virtual void addLine(IloTuplePathBuffer line);
	void removeValuesOnly(IloInt line);
private:
	void initReferencedColumns();
	IloTuplePattern makeTuplePattern(IloInt idx) const;

public:
	virtual IloMapItem getMapItem(IloInt idx) const;

	void setLine(IloInt, IloTupleCellArray line);
	IloDiscreteDataCollectionI* makeColumnAsSet(IloInt idx) const;
	IloInt getRealIndex(IloInt columnIdx, IloInt idx);
	IloBool hasReference() const { return _hasReference;}

	IloIntArray getReferencedColumns() const{
		IloTestAndRaise(_referencedColumns.getImpl(), "IloTupleCollectionI::getReferencedColumns should not be called");
		return _referencedColumns;
	}
	IloIntArray getReferenceKeyIndexes() const{
		IloTestAndRaise(_referenceKeyColumns.getImpl(), "IloTupleCollectionI::getReferenceKeyIndexes should not be called");
		return _referenceKeyColumns;
	}
	void setReferences(IloIntArray idx, IloArray<IloDiscreteDataCollectionI*> references);

	IloAnyArray makePointersToColumns() const;
	IloIntArray getCollectionColumnIndexes() const{ return _collectionColumnIdx;}

	void fillWithDefaultValue(IloTupleCellArray cells);
	void fillWithDefaultValue(IloTupleBufferI* buf);
	void setDefaultValue(IloTupleCellArray cells);
	void setDefaultValue(IloTupleBufferI* buf);

	IloInt getInternalId(IloIntArray path) const;
	IloInt getInternalSubId(IloIntArray path) const;
	IloInt getReferenceKeyedInternalId(IloIntArray path) const;
	void fillHashForTupleCollection();
	void createHashForTupleCollection();
	void deleteHashForTupleCollection();

	IloTupleCells2ArrayIndexHash* getHashForTupleCollection() {
		if(!_hashForTupleCollection) {
			createHashForTupleCollection();
		}
		return _hashForTupleCollection;
	}

	IloAnyArray getSharedPointersToColumns() const{ return _pointers;}
	void setSize(IloInt size);
	friend class IloTupleIterator;
	/* Methodes pour Chris. */
	IloTupleCollectionI(IloEnv env, IloTupleSchema schema, IloInt numberOfLine, IloTupleCellArray defaultValue);
	IloTupleCollectionI(IloEnv env, IloTupleSchema schema, IloInt numberOfLine); /* initialise n lignes a 0. */
	IloTupleCollectionI(IloTupleCollectionI* set); /* par copie des data du tupleset. */
	IloDataCollectionArray getColumnArray() const { return _array; }
	virtual IloDataCollectionI* copy() const;

	void clearSelectIndexes();
	void createSelectIndexes();
	void createSelectIndexes(IloInt idxCol);
	void createSelectIndexes(IloIntArray idxCol);

private:
	IloDiscreteDataCollectionI* findColumn(const char* name) const;

private:
	void setValue(IloDiscreteDataCollection column, IloInt index, IloIntCollectionI* value);
	void setValue(IloDiscreteDataCollection column, IloInt index, IloNumCollectionI* value);
	void setValue(IloDiscreteDataCollection column, IloInt index, IloAnyCollectionI* value);

	void setValue(IloDiscreteDataCollection column, IloInt index, IloInt value);
	void setValue(IloDiscreteDataCollection column, IloInt index, IloNum value);
	void setValue(IloDiscreteDataCollection column, IloInt index, IloAny value);

	void setValue(IloIntArray path, IloInt index, IloInt value){
		setValue(getColumn(path), index, value);
	}
	void setValue(IloIntArray path, IloInt index, IloNum value){
		setValue(getColumn(path), index, value);
	}
	void setValue(IloIntArray path, IloInt index, IloAny value){
		setValue(getColumn(path), index, value);
	}

	void setValue(IloIntArray path, IloInt index, IloIntCollectionI* value){
		setValue(getColumn(path), index, value);
	}
	void setValue(IloIntArray path, IloInt index, IloNumCollectionI* value){
		setValue(getColumn(path), index, value);
	}
	void setValue(IloIntArray path, IloInt index, IloAnyCollectionI* value){
		setValue(getColumn(path), index, value);
	}

public:
	virtual void setLockable(IloBool flag);
	IloBool isKeyEnabled() const {
		return (this->getDataType() != TupleCollection)
			&&
			getSchema().getImpl()->getTotalColumnNumber() != getSchema().getImpl()->getOrMakeTotalKeyIndexes().getSize();
	}

protected:
    IloTupleCellArray makeKeyCells(IloInt line);
	IloTupleCellArray makeKeyCells(IloTupleCellArray);
public:
	IloTupleCellArray getOrMakeSharedTupleCells(IloTuplePatternI*);
protected:
	IloTupleCellArray getOrMakeInternalTupleCells(IloInt);

public:
    IloTupleBufferI* getOrMakeSharedTupleBuffer(){
        if ( !_sharedBuffer ) {
            delete _sharedBuffer;
            _sharedBuffer = makeTupleBuffer().getImpl();
        }
        _sharedBuffer->setIndex(-1);
        return _sharedBuffer;
    }

    IloTupleCellArray getOrMakeSharedTupleCells(IloTuplePathBuffer);
	IloTupleCellArray getOrMakeEmptySharedTupleCells();
	IloTupleCellArray getOrMakeSharedTupleCells(IloInt line);

	IloTupleCellArray getOrMakeSharedKeyCells(IloInt line);
	IloTupleCellArray getOrMakeSharedKeyCells(IloTupleCellArray);
	IloTupleCellArray getOrMakeEmptySharedKeyCells();

	IloTupleCellArray getOrMakeEmptyRefSliceCells();
	IloTupleCellArray getOrMakeEmptyReferenceCells();
	IloTupleCellArray getOrMakeSharedReferenceCells(IloTuplePathBuffer);

public:

	IloInt getAbsoluteIndex(IloTupleCellArray cells, IloInt pos) const;

	IloIntArray getSharedPathFromAbsolutePosition(IloInt position) const;

	virtual IloInt commit(IloTuplePatternI* exp);
	void commitWithoutSize(IloTupleCellArray array, IloInt startIndex);
public:
	virtual IloInt commit(IloTupleCellArray array);
	//void setLine(IloInt index, IloTuplePathBuffer buffer);
	virtual void remove(IloInt index);
	void remove(IloTuple buffer);
	virtual void remove(IloTupleCellArray buffer);

	void fillTupleCellsFromIndex(IloTupleCellArray array, IloInt line, IloInt index = 0) const;
	void fillTupleKeysFromIndex(IloTupleCellArray array, IloInt line, IloInt index = 0) const;


	void addTupleCells(IloTupleCellArray array, IloInt line) const;
	void addTupleBuffer(IloTupleBufferI* array, IloInt line) const;
	void addTupleKeys(IloTupleCellArray array, IloInt line) const;
	IloTupleCellArray makeTupleCells(IloTuplePathBuffer buf) const;
	//IloTupleCellArray makeTupleCells(IloInt line) const;
	//ok with references
	IloTupleCellArray makeTupleCells(IloInt line) const {
		// AFC 10/09/04 use the new addTupleCells that shouldbe more efficient.
		if (!hasReference()){
			IloTupleCellArray array(getEnv(), _pointers.getSize());
			for (IloInt i = 0; i < _pointers.getSize(); i++){
				IloDiscreteDataCollectionI* col = (IloDiscreteDataCollectionI*)_pointers[i];
				switch (col->getDataType()){
			  case IntDataColumn:
				  array[i] = IloTuplePath::IloTupleCell(((IloIntDataColumnI*)_pointers[i])->getArray()[line]);
				  break;
			  case NumDataColumn:
				  array[i] = IloTuplePath::IloTupleCell(((IloNumDataColumnI*)_pointers[i])->getArray()[line]);
				  break;
			  default:
				  array[i] = IloTuplePath::IloTupleCell(((IloAnyDataColumnI*)_pointers[i])->getArray()[line]);
				  break;
				}
			}
			return array;
		}
		IloTupleCellArray array(getEnv());
		addTupleCells(array, line);
		return array;
	}



	const char* getColumnName(IloInt index) const{
		return this->getColumn(index).getImpl()->getName();
	}

	IloDiscreteDataCollection getColumn(IloInt index) const{
		return (IloDiscreteDataCollectionI*)_array[index].getImpl();
	}

	IloTupleCollectionI* getTupleColumn(IloInt colIndex) const;
	IloTupleCollectionI* getTupleColumn(IloIntArray path) const;
	IloTupleCollectionI* getTupleColumn(IloInt size, IloInt* path) const;

	IloInt getColumnIndex(const char* name) const;
	IloInt getColumnIndex(const IloSymbolI* s1) const;

	virtual IloBool supportDuplicates() const{ return IloTrue; }

	IloTupleSchema getSchema() const { return _schema;}

	IloInt getLength() const {
		return _array.getSize();
	}

	virtual IloBool includes(IloDataCollectionI* dom) const;
	virtual IloDataType getDataType() const{
		return TupleCollection;
	}
	virtual void displayType(ILOSTD(ostream)& os) const;
	virtual IloAnyArray getArray() const;
	virtual IloBool isTupleCollection() const;

	IloInt getSize() const{
		return _size;
	}

	virtual ~IloTupleCollectionI();

	IloDiscreteDataCollection getColumn(IloIntArray path) const{
		IloInt colIdx = this->getInternalId(path);
		return (IloDiscreteDataCollectionI*)_pointers[colIdx];
	}


	IloTuple makeTuple(IloInt index) const{
		return new (getEnv()) IloTupleI((IloTupleCollectionI*)this, index);
	}

	IloTupleBuffer makeTupleBuffer(IloInt index = -1) const{
		return new (getEnv()) IloTupleBufferI((IloTupleCollectionI*)this, index);
	}

	IloTuplePathBuffer makeLine(IloInt index) const;
	void fillLine(IloTuplePathBuffer buf, IloInt index) const;
	virtual IloInt commit(IloTupleBuffer line);


	virtual void display(ILOSTD(ostream)& os) const;
	virtual void displayRow(IloInt i, ILOSTD(ostream)& out) const;
	virtual void displayAt(IloInt index, ILOSTD(ostream)& out) const {
		displayRow(index, out);
	}
	IloDataIterator* iterator(IloGenAlloc* heap) const;

	IloDataIterator* iterator() const { return this->iterator(getEnv()->getGeneralAllocator()); }

	virtual IloInt getIndex(IloSymbol value) const;
	virtual IloInt getIndex(IloInt value) const;
	virtual IloInt getIndex(IloNum value) const;

	virtual IloInt getIndex(IloAny value) const; /* provient de IloAnycollectionI... */
	virtual IloBool contains(IloAny elt) const;/* provient de IloAnycollectionI... */
	virtual IloAny getValue(IloInt index) const;/* provient de IloAnycollectionI... */

	virtual IloBool areEquivalents(IloInt index1, IloInt index2) const;
	IloTupleSetI* makeKeySet() const;
};




class IloTupleSetI : public IloTupleCollectionI {
	ILORTTIDECL
		virtual IloInt commit(IloTupleCellArray array){
			return commit(array, IloTrue);
		}
protected:
	IloInt commit2HashTable(IloTupleCellArray array, IloBool check);
	void fillColumns();

public:
	friend class IloTupleIterator;
public:
	//virtual void setReferences(IloIntArray idx, IloArray<IloDiscreteDataCollectionI*> references);

	IloAnyArray makePointersToKeyColumns() const;

protected:
	//enum Type { INT=1, NUM=2, ANY=3, SYMBOL=4};
	//IloTupleSetI(IloEnv env, Type type, IloInt arity);
	IloVBHash* _VBhash;
	IloVBHash* _UnaryVBhash;
	IloTupleCells2IndexHash* _tupleHash;

private:
	IloDiscreteDataCollectionI* findColumn(const char* name) const;
	IloIntArray makeTheQuery(IloTupleRequest query, IloBool& arrayCreated);

public:
	virtual IloInt getMapItemIndex(IloMapItem value) const;

	virtual IloTupleCollectionChangeNotifierI* getChangeNotifier();
	IloTupleSetI* makeTupleSetKey() const;

	void build(IloExtendedComprehensionI* comp, IloModelEvaluatorI* eval=0);
	IloTupleCells2IndexHash* getTupleCellsHash() const {
		if (_tupleHash)
			return _tupleHash;
		((IloTupleSetI*)this)->createTupleHash();
		return _tupleHash;
	}
	IloIntArray select(IloTupleRequest query, IloBool& arrayCreated);

	IloEnvObjectI* findVB(IloRequestSignatureI* req);
	IloEnvObjectI* findUnaryVB(IloRequestSignatureI* req);
	void addVB(IloRequestSignatureI* req, IloEnvObjectI* a);
	void addUnaryVB(IloRequestSignatureI* req, IloEnvObjectI* a);

	IloTupleBuffer checkUnicityOfTuple(IloTupleRequest query);

private:
	// This method mat return null
	//ok with references.
	IloIntArray select(IloInt value, IloDiscreteDataCollection column, IloBool& arrayCreated, IloTuplePath::IloTupleRequestType reqType){
		IloDiscreteDataCollectionI* coll = column.getImpl();
		if (reqType == IloTuplePath::Equality) {
			return ((IloIntDataColumnI*)coll)->find(value, arrayCreated);
		} else if (reqType == IloTuplePath::LowerBound) {
			return ((IloIntDataColumnI*)coll)->findGe(value, arrayCreated);
		} else if (reqType == IloTuplePath::UpperBound) {
			return ((IloIntDataColumnI*)coll)->findLe(value, arrayCreated);
		}
		throw IloWrongUsage("IloIntArray IloTupleSetI::select");
		return 0;
	}

	// This method may return null
	//ok with references.
	IloIntArray select(IloNum value, IloDiscreteDataCollection column, IloBool& arrayCreated, IloTuplePath::IloTupleRequestType reqType){
		IloDiscreteDataCollectionI* coll = column.getImpl();
		if (reqType == IloTuplePath::Equality) {
			return ((IloNumDataColumnI*)coll)->find(value, arrayCreated);
		} else if (reqType == IloTuplePath::LowerBound) {
			return ((IloNumDataColumnI*)coll)->findGe(value, arrayCreated);
		} else if (reqType == IloTuplePath::UpperBound) {
			return ((IloNumDataColumnI*)coll)->findLe(value, arrayCreated);
		}
		throw IloWrongUsage("IloIntArray IloTupleSetI::select");
		return 0;
	}

	// This method may return null
	//ok with references.
	IloIntArray select(IloAny value, IloDiscreteDataCollection column, IloBool& arrayCreated){
		IloDiscreteDataCollectionI* coll = column.getImpl();
		return ((IloAnyDataColumnI*)coll)->find(value, arrayCreated);
	}


private:
	//void addLine(IloTuplePathBuffer buffer);

	IloIntArray select(IloTuplePath* path, IloBool& arrayCreated);

	// This method may return null
	IloIntArray select(IloInt value, IloIntArray path, IloBool& arrayCreated, IloTuplePath::IloTupleRequestType reqType) {
		return select(value, getColumn(path), arrayCreated, reqType);
	}
	// This method may return null
	IloIntArray select(IloNum value, IloIntArray path, IloBool& arrayCreated, IloTuplePath::IloTupleRequestType reqType) {
		return select(value, getColumn(path), arrayCreated, reqType);
	}
	// This method may return null
	IloIntArray select(IloAny value, IloIntArray path, IloBool& arrayCreated) {
		return select(value, getColumn(path), arrayCreated);
	}

    //IloIntArray select(IloInt value, IloIntArray path, IloBool& arrayCreated, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	//IloIntArray select(IloNum value, IloIntArray path, IloBool& arrayCreated, IloTuplePath::IloTupleRequestType = IloTuplePath::Equality);
	//IloIntArray select(IloAny value, IloIntArray path, IloBool& arrayCreated);

public:
	IloTupleSetI(IloTupleSetI* set);
	virtual IloTuple makeNext(IloTuple value, IloInt n=0) const;
	virtual IloTuple makePrevious(IloTuple value, IloInt n=0) const;

	virtual IloTuple makeNextC(IloTuple value, IloInt n=0) const;
	virtual IloTuple makePreviousC(IloTuple value, IloInt n=0) const;

	virtual IloTuple makeFirst() const;
	virtual IloTuple makeLast() const;

	virtual IloBool supportDuplicates() const;

	IloBool isIn(IloTupleBuffer buffer);
	IloBool isIn(IloTuplePathBuffer buffer);

	IloInt getIndex(IloTupleCellArray array);
	IloInt getIndex(IloTuplePatternI* exp);
	IloTupleI* find(IloTupleBufferI* buffer);
	IloTupleI* find(IloTupleBuffer buffer);
	IloTupleI* find(IloTuplePathBuffer buffer);

	void fillTupleHash();
	void createTupleHash();
	void clearTupleHash();

  void clearAllCaches();


	void createHashForSelect();

	void deleteHashForSelect(){
		if (_VBhash){
			for (IloVBHash::Iterator it(_VBhash); it.ok(); ++it){
				// Request signature are shared with the other hash -> we delete it in the other hash.
				//IloRequestSignatureI* sel = it.getKey();
				//delete sel; sel=0;
				IloEnvObjectI* hash = (IloEnvObjectI*)*(it);
				delete hash; hash = 0;
			}
			delete _VBhash; _VBhash = 0;
		}
		if (_UnaryVBhash){
			for (IloVBHash::Iterator it(_UnaryVBhash); it.ok(); ++it){
				IloRequestSignatureI* sel = it.getKey();
				delete sel; sel=0;
				IloEnvObjectI* hash = (IloEnvObjectI*)*(it);
				delete hash; hash = 0;
			}
			delete _UnaryVBhash; _UnaryVBhash = 0;
		}
	}
	IloBool isHashForSelectCreated() const {
		return (_VBhash != 0 && _UnaryVBhash != 0);
	}

	IloIntDataColumnI* getIntColumn(IloInt colIndex) const;
	IloIntDataColumnI* getIntColumn(const char* name) const;

	IloNumDataColumnI* getNumColumn(IloInt colIndex) const;
	IloNumDataColumnI* getNumColumn(const char* name) const;

	IloAnyDataColumnI* getAnyColumn(IloInt colIndex) const;
	IloAnyDataColumnI* getAnyColumn(const char* name) const;


	IloIntCollectionColumnI* getIntCollectionColumn(IloInt colIndex) const;
	IloIntCollectionColumnI* getIntCollectionColumn(const char* name) const;

	IloNumCollectionColumnI* getNumCollectionColumn(IloInt colIndex) const;
	IloNumCollectionColumnI* getNumCollectionColumn(const char* name) const;

	IloAnyCollectionColumnI* getAnyCollectionColumn(IloInt colIndex) const;
	IloAnyCollectionColumnI* getAnyCollectionColumn(const char* name) const;


	virtual IloDataType getDataType() const{
		return TupleSet;
	}
	virtual void displayType(ILOSTD(ostream)& os) const;
	virtual IloBool isTupleSet() const;

	IloTupleSetI(IloEnv env, IloTupleSchema schema);
	//IloTupleSetI(IloEnv env, IloTupleSchema schema, IloTupleBufferI* defaultValue);
	IloTupleSetI(IloEnv env, IloTupleSchema schema, IloTupleCellArray defaultValue);

	virtual ~IloTupleSetI();


	void enableSelectIndexes(){
		//createHashForSelect();
		enableSelectIndexesOnColumn();
	}

	void disableSelectIndexes(){
		disableSelectIndexesOnColumn();
	}

	void setColumnName(IloIntArray path, const char* name){
		getColumn(path).getImpl()->setName(name);
	}
	const char* getColumnName(IloIntArray path) const{
		return this->getColumn(path).getImpl()->getName();
	}

	void setColumnName(IloInt index, const char* name){
		this->getColumn(index).getImpl()->setName(name);
	}
	const char* getColumnName(IloInt index) const{
		return this->getColumn(index).getImpl()->getName();
	}

	virtual IloInt commit(IloTupleBuffer line);
	IloInt commit(IloTupleCellArray line, IloBool check);

	IloInt commit(IloTupleBuffer line, IloBool check);
	IloInt commit(IloInt index, IloTuplePathBuffer line, IloBool check);

	virtual IloDataIterator* iterator(IloGenAlloc* heap) const;
	virtual IloDataIterator* iterator() const { return this->iterator(getEnv()->getGeneralAllocator()); }

	virtual IloInt getIndex(IloInt value) const;
	virtual IloInt getIndex(IloNum value) const;
	virtual IloInt getIndex(IloSymbol value) const;
	virtual IloInt getIndex(IloAny value) const;
	IloInt getIndex(IloTupleI* tuple) const;
	IloInt getIndex(IloTuple tuple) const;

	virtual IloInt commit(IloTuplePatternI*);
	IloInt commit(IloTuplePatternI* exp, IloBool check);
	virtual IloDataCollectionI* copy() const;
	virtual void remove(IloInt index);
	//virtual void remove(IloTuple buffer);
	virtual void remove(IloTupleCellArray buffer);
	virtual IloBool areEquivalents(IloInt index1, IloInt index2) const;
	IloDiscreteDataCollectionI* smartCopy() const;
};


//------------------


class IloTupleSetExprOperatorI : public IloTupleExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _coll;
	IloTupleExprI* _expr;
	IloIntExprI* _offset;
	IloBool _circ;
	IloBool _sense;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getTupleCollection() const { return _coll; }
	IloTupleExprI* getTupleExpr() const { return _expr; }
	IloIntExprI* getOffSet() const { return _offset; }
	IloBool isCircular() const { return _circ; }
	IloBool isNext() const { return _sense; }
	IloTupleSetExprOperatorI(IloEnvI* env,
		IloTupleSetExprI* coll,
		IloTupleExprI* index,
		IloIntExprI* offset = 0,
		IloBool sense = IloTrue,
		IloBool circ = IloTrue);
	virtual ~IloTupleSetExprOperatorI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};


class IloTupleSetExprItemI : public IloTupleExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _coll;
	IloIntExprI* _offset;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getTupleCollection() const { return _coll; }
	IloIntExprI* getOffSet() const { return _offset; }
	IloTupleSetExprItemI(IloEnvI* env,
		IloTupleSetExprI* coll,
		IloIntExprI* offset = 0);
	virtual ~IloTupleSetExprItemI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};


class IloTupleSetOrdI : public IloIntExprI {
  ILOEXTRDECL
  IloTupleSetExprI* _coll;
  IloTupleExprI* _expr;
  virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTupleSetOrdI(IloEnvI* env, IloTupleSetExprI* coll, IloTupleExprI* expr) :
    IloIntExprI(env), _coll((IloTupleSetExprI*)coll->lockExpr()), _expr(expr->lockExpr()) {}
  virtual ~IloTupleSetOrdI();
  void display(ILOSTD(ostream)& out) const;
  virtual IloNum eval(const IloAlgorithm alg) const;
  virtual IloExtractableI* makeClone(IloEnvI*) const;
  IloTupleSetExprI* getCollection() const { return _coll; }
  IloTupleExprI* getExpr() const { return _expr; }
};




class IloTupleCollectionFirstI : public IloTupleExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _coll;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getExpr() const { return _coll; }
	IloTupleCollectionFirstI(IloEnvI* env, IloTupleSetExprI* coll);
	virtual ~IloTupleCollectionFirstI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

class IloTupleCollectionLastI : public IloTupleExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _coll;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getExpr() const { return _coll; }
	IloTupleCollectionLastI(IloEnvI* env, IloTupleSetExprI* coll);
	virtual ~IloTupleCollectionLastI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};

class IloTupleCollectionInterI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _left;
	IloTupleSetExprI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getLeft() const { return _left; }
	IloTupleSetExprI* getRight() const { return _right; }
	IloTupleCollectionInterI(IloEnvI* env, IloTupleSetExprI* left, IloTupleSetExprI* right);
	virtual ~IloTupleCollectionInterI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};


//========================
class IloTupleCollectionUnionI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _left;
	IloTupleSetExprI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getLeft() const { return _left; }
	IloTupleSetExprI* getRight() const { return _right; }
	IloTupleCollectionUnionI(IloEnvI* env, IloTupleSetExprI* left, IloTupleSetExprI* right);
	virtual ~IloTupleCollectionUnionI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};


//======================
class IloTupleCollectionExcludeI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _left;
	IloTupleSetExprI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getLeft() const { return _left; }
	IloTupleSetExprI* getRight() const { return _right; }
	IloTupleCollectionExcludeI(IloEnvI* env, IloTupleSetExprI* left, IloTupleSetExprI* right);
	virtual ~IloTupleCollectionExcludeI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};


//===========================
class IloTupleCollectionSymExcludeI : public IloTupleSetExprI {
	ILOEXTRDECL
private:
	IloTupleSetExprI* _left;
	IloTupleSetExprI* _right;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	IloTupleSetExprI* getLeft() const { return _left; }
	IloTupleSetExprI* getRight() const { return _right; }
	IloTupleCollectionSymExcludeI(IloEnvI* env, IloTupleSetExprI* left, IloTupleSetExprI* right);
	virtual ~IloTupleCollectionSymExcludeI();
	void display(ILOSTD(ostream)& out) const;
	virtual IloNum eval(const IloAlgorithm alg) const;
	virtual IloExtractableI* makeClone(IloEnvI*) const;
	IloEncoderI* getEncoder() const;
};


class IloTupleCollectionCardI : public IloIntExprI {
	ILOEXTRDECL
		IloTupleSetExprI* _expr;
	virtual void visitSubExtractables(IloExtractableVisitor* v);
public:
	/* <p> . </p> */
	IloTupleCollectionCardI(IloEnvI* env, IloTupleSetExprI* expr) :
	  IloIntExprI(env), _expr((IloTupleSetExprI*)expr->lockExpr()) {}
	  virtual ~IloTupleCollectionCardI();
	  /* <p> . </p> */
	  void display(ILOSTD(ostream)& out) const;
	  /* <p> . </p> */
	  virtual IloNum eval(const IloAlgorithm alg) const;
	  /* <p> . </p> */
	  virtual IloExtractableI* makeClone(IloEnvI*) const;
	  /* <p> . </p> */
	  IloTupleSetExprI* getExpr() const { return _expr; }
};

class IloTupleSetConstI : public IloTupleSetExprI {
  ILOEXTRDECL
private:
  IloTupleSetI* _tuple;
  IloBool _ownsColl;
  void visitSubExtractables(IloExtractableVisitor* v);
public:
  IloTupleSetConstI(IloEnvI* env, IloTupleSetI* tuple, IloBool ownsTuple=IloFalse);
  virtual ~IloTupleSetConstI();
  IloTupleSetI* getTupleSet() const { return _tuple; }
  virtual IloEncoderI* getEncoder() const {
    return getEnv()->getTupleEncoder();
  }
  ILOEXTROTHERDECL
};


#endif
