// ----------------------------------------------------------*- C++ -*-------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

#ifndef __XML_iloxmlabstractH
#define __XML_iloxmlabstractH

#if !defined(__CONCERT_ilosysH)
#include <ilconcert/ilosys.h>
#endif

#if !defined(__XML_iloreaderH)
#include <ilconcert/iloreader.h>
#endif

#if !defined(__CONCERT_iloexpressioniH)
#include <ilconcert/iloexpressioni.h>
#endif

#if !defined(__CONCERT_ilopathiH)
# include <ilconcert/ilopathi.h>
#endif

#if !defined(__CONCERT_ilosetiH)
# include <ilconcert/iloseti.h>
#endif
#if !defined(__CONCERT_ilodiffiH)
# include <ilconcert/ilodiffi.h>
#endif

#if !defined(__XML_ilowriterH)
#include <ilconcert/ilowriter.h>
#endif

#if !defined(__XML_ilotagH)
#include <ilconcert/ilxml/ilotag.h>
#endif


#ifdef ILO_RS6000
void* operator new (size_t sz, const IloEnv& env);
void* operator new[] (size_t s, const IloEnv& env);
#endif

#define IloDistributeVI IloDistributeI


#if !defined(__XML_iloxmlcontextH)
#include <ilconcert/iloxmlcontext.h>
#endif

#if !defined(__CONCERT_ilomapH)
# include <ilconcert/ilomap.h>
#endif

class IloXmlElement;


class IloConstraintI;
class IloIntExprI;
class IloNumExprI;

class IloMapIntIndexI;
class IloMapIntExprIndexI;
class IloMapNumIndexI;
class IloMapNumExprIndexI;


template <class XMLINFO, class EXPR> IloBool ITERATE(IloXmlReader reader, IloXmlElement* element, XMLINFO* info, EXPR* exprI){
  for (IloXmlListIteratorOf(IloXmlNodeI) i(element->getChildren());
    i.hasMoreElements();) {
      IloXmlNodeI* node = i.getNextElement();
      if (node->getType() == IloXmlNodeI::Element) {
        IloXmlElement* subElement = (IloXmlElement*)node;
        if (!info->checkSubElement(reader, subElement, exprI)){
          return IloFalse;
        }
      }
    }
    return IloTrue;
}


template <class XMLINFO, class EXPR> IloRtti* CREATE_ITERATOR_TERM(IloXmlReader reader, IloXmlElement* element, XMLINFO* info, EXPR* exprI){
  exprI = new (reader.getEnv()) EXPR(reader.getEnvImpl(),
  reader.readAttribute(element, IloXmlAttributeDef::Name));
  if (ITERATE(reader, element, info, exprI)){
    return (IloRtti*)exprI;
  }
  return 0;
}


template <class XMLINFO, class EXPR> IloExtractableArray* CREATE_ITERATOR_TERM_FOR_ARRAY(IloXmlReader reader, IloXmlElement* element, XMLINFO* info, EXPR* exprI){
  exprI = new (reader.getEnv()) EXPR(reader.getEnv(),0);
  ITERATE(reader, element, info, exprI);
  return (IloExtractableArray*)exprI;
}


class IloXmlInfo {
protected:
  char* _version;
  IloXmlContextI* _context;
public:

  virtual ~IloXmlInfo();
public:
  IloXmlInfo(IloXmlContextI* context, const char* version=0);

public:
  IloXmlInfo(){}

public:
  static const char* getTagName();

public:
  virtual const char* getTag();
 
public:
  virtual IloXmlElement* getTagElement(IloXmlWriter writer, const IloRtti* exprI);


  IloXmlContextI* getContext() { return _context; }

protected:
  const char* getVersion();

  void setVersion(const char* version);

public:
  IloBool checkExprExistence(IloXmlReader reader,
    IloXmlElement* element,
    const char* attribute,
    IloInt& id);
public:
  IloBool getRefInChild(IloXmlReader reader,
    IloXmlElement* element,
    IloInt& id);


public:
  IloBool checkAttExistence(IloXmlReader reader,
    IloXmlElement* element,
    const char* attribute);

public:
  IloBool writeRtti(IloXmlWriter writer, IloXmlElement* element, const IloRtti* rtti, const char* attribute = 0);

public:
  IloBool writeExtractable(IloXmlWriter writer, IloXmlElement* element, const IloExtractable extractable, const char* attribute = 0){
     return writeRtti(writer, element, extractable.getImpl(), attribute);
  }

public:
  void writeSolutionValue(IloXmlWriter writer, const IloSolution solution, IloXmlElement* element, const IloRtti* rtti, const char* attribute);

public:
  virtual IloBool writeXml(IloXmlWriter writer, const IloExtractableI* exprI, IloXmlElement* masterElement);

public:
  virtual IloBool write(IloXmlWriter writer, const IloRtti* exprI, IloXmlElement* masterElement);


public:
  virtual IloBool writeRef(IloXmlWriter writer, const IloRtti* exprI, IloXmlElement* masterElement);

public:
  virtual IloBool writeXmlRef(IloXmlWriter writer, const IloExtractableI* exprI, IloXmlElement* masterElement);

public:
  IloBool readExtractable(IloXmlReader reader, IloXmlElement* element){
    return readRtti(reader, element);
  }

public:
  IloBool readRtti(IloXmlReader reader, IloXmlElement* element);

protected:
  virtual IloRtti* read(IloXmlReader reader, IloXmlElement* element);

protected:
  virtual IloExtractableI* readXml(IloXmlReader reader, IloXmlElement* element);


public:
  virtual IloExtractableI* readFromXml(IloXmlReader reader, IloXmlElement* element);

public:
  virtual IloRtti* readFrom(IloXmlReader reader, IloXmlElement* element);

protected:
  virtual IloBool checkSubElement(IloXmlReader reader, IloXmlElement* subElement, IloRtti* exprI);

protected:
 /*
  * @deprecated
  */
  virtual IloBool checkSubElement(IloXmlReader reader, IloXmlElement* subElement, IloExtractableI* exprI);

protected:
  virtual IloExtractableArray* readXmlArray(IloXmlReader reader, IloXmlElement* element);
  virtual IloMapI* readXmlMap(IloXmlReader reader, IloXmlElement* element);

public:
  virtual IloExtractableArray* readArrayFromXml(IloXmlReader reader, IloXmlElement* element);

public:
  IloXmlElement* setCommonArrayXml(IloXmlWriter writer,
    const IloExtractableArray* extractable);

protected:
  virtual IloBool checkSubElement(IloXmlReader reader,
    IloXmlElement* subElement,
    IloExtractableArray* varArray);

public:
  virtual int write(IloXmlWriter writer, const IloExtractableArray* extractable, IloXmlElement* masterElement);

public:
  IloXmlElement* setCommonXml(IloXmlWriter writer, const IloRtti* exprI);

public:
  IloXmlElement* setCommonValueXml(IloXmlWriter writer, const IloRtti* exprI);

public:
  void setXml(IloXmlWriter writer, IloXmlElement* element, const IloRtti* exprI);

protected:
  IloXmlElement* setNumArray(IloXmlWriter writer, const IloNumArray Array);

protected:
  IloXmlElement* setIntSet(IloXmlWriter writer, const IloIntSet Array);


protected:
  IloXmlElement* setNumSet(IloXmlWriter writer, const IloNumSet Array);

  IloXmlElement* setIntRange(IloXmlWriter writer, const IloIntRange Array);

protected:
  IloXmlElement* setIntArray(IloXmlWriter writer, const IloIntArray Array);

protected:
  IloXmlElement* setBoolArray(IloXmlWriter writer, const IloBoolArray Array);

protected:
  IloNumVar::Type getVarType(IloXmlReader reader, IloXmlElement* element);

protected:
  IloBool getNumValArray(IloXmlReader reader, IloXmlElement* element, IloNumArray& numArray);

protected:
  IloBool getIntValArray(IloXmlReader reader, IloXmlElement* element, IloIntArray& intArray);

public:
  virtual IloBool readSolution(IloXmlReader reader, IloSolution solution, IloXmlElement* element);

protected:
  IloBool writeVarArray(IloXmlWriter writer,
    IloXmlElement* element,
    IloIntVarArray array);

protected:
  IloBool writeVarArray(IloXmlWriter writer,
    IloXmlElement* element,
    IloBoolVarArray array);

protected:
  IloBool writeVarArray(IloXmlWriter writer,
    IloXmlElement* element,
    IloIntExprArray array);

protected:
  IloBool writeVarArray(IloXmlWriter writer,
    IloXmlElement* element,
    IloNumExprArray array);

protected:
  IloBool writeVarArray(IloXmlWriter writer,
    IloXmlElement* element,
    IloIntSetVarArray array);

protected:
  IloBool writeVarArray(IloXmlWriter writer,
    IloXmlElement* element,
    IloNumVarArray array);

protected:
  IloBool writeVarArray(IloXmlWriter writer,
    IloXmlElement* element,
    IloRangeArray array);

protected:
  IloBool write(IloXmlWriter writer, IloXmlElement* element, IloIntMap array);
  IloBool write(IloXmlWriter writer, IloXmlElement* element, IloIntExprMap array);
  IloBool write(IloXmlWriter writer, IloXmlElement* element, IloNumMap array);
  IloBool write(IloXmlWriter writer, IloXmlElement* element, IloNumExprMap array);

protected:
  IloBool writeVarArray(IloXmlWriter writer,
    IloXmlElement* element,
    IloConstraintArray array);

protected:
  IloBool writeVarArray(IloXmlWriter writer,
    IloXmlElement* element,
    IloSemiContVarArray array);

protected:
  IloBool writeVarArray(IloXmlWriter writer,
    IloXmlElement* element,
    IloSOS1Array array);

protected:
  IloBool writeVarArray(IloXmlWriter writer,
    IloXmlElement* element,
    IloSOS2Array array);


public:
  virtual void writeSolution(IloXmlWriter writer,
    const IloSolution solution,
    const IloExtractable extractable);
};

class IloSerializeError : public IloException {
public:
  IloSerializeError(const char* message);
  virtual ~IloSerializeError();
};




#endif
