// -------------------------------------------------------------- -*- C++ -*-
// File: ilconcert/ilodatasourcei.h
// --------------------------------------------------------------------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

// -*- C++ -*- header file.
#ifndef __CONCERT_ilodatasourceiH
#define __CONCERT_ilodatasourceiH

#ifndef __CONCERT_ilocsvreaderH
#include <ilconcert/ilocsvreader.h>
#endif
#ifndef __CONCERT_ilosymbolH
#include <ilconcert/ilosymbol.h>
#endif
#ifndef __CONCERT_iloanysetH
#include <ilconcert/iloanyset.h>
#endif

#ifndef __CONCERT_ilotuplecollectionH
#include <ilconcert/ilotuplecollection.h>
#endif
#ifndef __CONCERT_ilotupleH
#include <ilconcert/ilotuple.h>
#endif


#ifdef ILO_MSVC
#if defined(ILO_WIN32) ||  (defined(ILO_MSVC8) && !defined(_IA64_))
#ifndef __CONCERT_ilodrvsheetH
#include <ilconcert/ilsource/ilodrvsheet.h>
#endif

#include <stdio.h>
#include <wtypes.h>
#include <comdef.h>
#endif
#endif

class IloCsvTableReaderI;
class IloCsvTableReader;

class IloCsvLineI;

class IloAbstractTupleMapI;


/**
* @group optim.concert.source
* @show internal
*
* <p>String DataSource.</p> 
*/

class IloStringDataSourceI : public IloRttiEnvObjectI{
	ILORTTIDECL
public:
	/** <p> . </p> */
	IloStringDataSourceI(IloEnv env);
	~IloStringDataSourceI();

	/** <p> . </p> */
	IloIntArray getIntArray(const char* str);
	/** <p> . </p> */
	IloNumArray getNumArray(const char* str);
	/** <p> . </p> */
	IloSymbolArray getSymbolArray(const char* str);

	/** <p> . </p> */
	IloIntSet getIntSet(const char* str);
	/** <p> . </p> */
	IloNumSet getNumSet(const char* str);
	/** <p> . </p> */
	IloSymbolSet getSymbolSet(const char* str);

	/** <p> . </p> */
	IloInt getInt(const char* str);
	/** <p> . </p> */
	IloNum getNum(const char* str);
	/** <p> . </p> */
	IloSymbol getSymbol(const char* str);

	/** <p> . </p> */
	IloIntArray2 getIntArray2(const char* str);
	/** <p> . </p> */
	IloNumArray2 getNumArray2(const char* str);
};

/**
* @group optim.concert.source
* @show internal
*
* <p>CSV DataSource.</p> 
*/
class IloCsvDataSourceI : public IloRttiEnvObjectI{
	ILORTTIDECL
private:
	void closeConnexion();
	IloCsvReader _reader;
public:
	/** <p> . </p> */
	IloCsvDataSourceI(IloEnv env, const char* fileName);
	~IloCsvDataSourceI();

	/** <p> . </p> */
	IloIntArray getIntArrayFromLine(const char* tableName, IloInt line);
	/** <p> . </p> */
	IloIntArray getIntArrayFromColumn(const char* tableName, IloInt column);
	/** <p> . </p> */
	IloIntSet getIntSetFromLine(const char* tableName, IloInt line);
	/** <p> . </p> */
	IloIntSet getIntSetFromColumn(const char* tableName, IloInt column);

	/** <p> . </p> */
	IloNumArray getNumArrayFromLine(const char* tableName, IloInt line);
	/** <p> . </p> */
	IloNumArray getNumArrayFromColumn(const char* tableName, IloInt column);
	/** <p> . </p> */
	IloNumSet getNumSetFromLine(const char* tableName, IloInt line);
	/** <p> . </p> */
	IloNumSet getNumSetFromColumn(const char* tableName, IloInt column);

	/** <p> . </p> */
	IloSymbolArray getSymbolArrayFromLine(const char* tableName, IloInt line);
	/** <p> . </p> */
	IloSymbolArray getSymbolArrayFromColumn(const char* tableName, IloInt column);
	/** <p> . </p> */
	IloSymbolSet getSymbolSetFromLine(const char* tableName, IloInt line);
	/** <p> . </p> */
	IloSymbolSet getSymbolSetFromColumn(const char* tableName, IloInt column);

	/** <p> . </p> */
	IloInt getIntFromTable(const char* tableName, IloInt x, IloInt y);
	/** <p> . </p> */
	IloNum getNumFromTable(const char* tableName, IloInt x, IloInt y);
	/** <p> . </p> */
	IloSymbol getSymbolFromTable(const char* tableName, IloInt x, IloInt y);

	/** <p> . </p> */
	IloTupleSet getTupleSet(IloTupleSchema schema, const char* tableName);

};

#ifdef ILO_MSVC
#if defined(ILO_WIN32) || defined(ILO_MSVC8)
/**
* @group optim.concert.source
* @show internal
*
* <p>Excel DataSource.</p>
*/
class IloExcelDataSourceI : public IloRttiEnvObjectI{
	ILORTTIDECL
private:
	void closeConnexion();
	IloExcelSpreadsheet _reader;
	IloInt getIntCell(IloInt x, IloInt y);
	IloIntArray readIntArray(IloEnvI* env, IloInt i, IloInt j);
	IloNumArray readNumArray(IloEnvI* env, IloInt i, IloInt j);
	IloAnyArray readSymbolArray(IloEnvI* env, IloInt i, IloInt j);
    void fillTupleSet(IloTupleCollectionI* set, IloInt i, IloInt j);

	void write(IloIntArray ar, IloInt i, IloInt j);
	void write(IloNumArray ar, IloInt i, IloInt j);
	void write(IloAnyArray ar, IloInt i, IloInt j);
	void write(IloTupleCollectionI* ar, IloInt i, IloInt j);
	IloNum getFloatCell(IloInt x, IloInt y);
	const char* getStringCell(IloInt x, IloInt y);
public:
	IloExcelDataSourceI(IloEnv env, const char* fileName, IloBool readOnly = IloFalse);
	~IloExcelDataSourceI();

	IloInt getIntFromTable(const char* range, IloInt x, IloInt y);
	IloNum getNumFromTable(const char* range, IloInt x, IloInt y);
	IloSymbol getSymbolFromTable(const char* range, IloInt x, IloInt y);

    /* @internal Used by YOPL. */
    void readIntMap(IloMapI* result, const char* range);
    /* @internal Used by YOPL. */
    void readNumMap(IloMapI* result, const char* range);
    /* @internal Used by YOPL. */
    void readSymbolMap(IloMapI* result, const char* range);
    /* @internal Used by YOPL. */
    void readTupleMap(IloAbstractTupleMapI* result, const char* range);

    /* @internal Used by YOPL. */
    void readIntSet(IloIntSetI* result, const char* range);

    /* @internal Used by YOPL. */
    void readNumSet(IloNumSetI* result, const char* range);

    /* @internal Used by YOPL. */
    void readSymbolSet(IloAnySetI* result, const char* range);

    /* @internal Used by YOPL. */
    void readTupleSet(IloTupleSetI* result, const char* range);


    /* @internal Used by YOPL. */
    void writeIntSet(IloIntSetI* set, const char* range);

    /* @internal Used by YOPL. */
    void writeNumSet(IloNumSetI* set, const char* range);

    /* @internal Used by YOPL. */
    void writeSymbolSet(IloAnySetI* set, const char* range);

    /* @internal Used by YOPL. */
    void writeTupleSet(IloTupleSetI* set, const char* range);



    /* @internal Used by YOPL. */
    void writeInt(IloInt val, const char* range);

    /* @internal Used by YOPL. */
    void writeNum(IloNum val, const char* range);

    /* @internal Used by YOPL. */
    void writeSymbol(IloSymbolI* val, const char* range);

    /* @internal Used by YOPL. */
    void writeTuple(IloTupleI* tuple, const char* range);



    /* @internal Used by YOPL. */
    void writeIntMap(IloMapI* set, const char* range);
    /* @internal Used by YOPL. */
    void writeNumMap(IloMapI* set, const char* range);
    /* @internal Used by YOPL. */
    void writeSymbolMap(IloMapI* set, const char* range);
    /* @internal Used by YOPL. */
    void writeTupleMap(IloAbstractTupleMapI* set, const char* range);

	void save();
};

#endif //ILO_WIN32
#endif //ILO_MSVC


#endif


