// -------------------------------------------------------------- -*- C++ -*-
// OPL Studio source file
// --------------------------------------------------------------------------
//  Copyright (C) 1999-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------

#ifndef __CONCERT_iloexceldrvH
#define __CONCERT_iloexceldrvH


#if !defined(__CONCERT_ilodataportH)
#include <ilconcert/ilsource/ilodataport.h>
#endif

#if defined(ILO_MSVC)
#if defined(ILO_WIN32) ||  (defined(ILO_MSVC8) && !defined(_IA64_))

#pragma  warning( disable : 4146 )
#import "ilconcert/ilsource/mso9.dll" no_namespace rename("DocumentProperties", "DocumentPropertiesXL") rename("RGB", "RBGXL")
#import "ilconcert/ilsource/vbe6ext.olb" no_namespace rename("_Windows", "_WindowsXL")
#import "ilconcert/ilsource/excel9.olb" rename("DialogBox", "DialogBoxXL") rename("RGB", "RBGXL") rename("DocumentProperties", "DocumentPropertiesXL") rename("CopyFile", "CopyFileXL") rename("ReplaceText", "ReplaceTextXL")


#if !defined(__CONCERT_iloenvH)
#include <ilconcert/iloenv.h>
#endif

class IloTupleCollectionI;
using namespace Excel;

/**
* @group optim.concert.source
* @show longernal
*
* <p>Excel longernal.</p>
*/
class IloExcelConnectionError : public IloException {
public:
  IloExcelConnectionError();
  IloExcelConnectionError(const wchar_t* message);
public:
  virtual ~IloExcelConnectionError();
};


class IloExcelReadOnlyError : public IloException {
public:
  IloExcelReadOnlyError();
public:
  virtual ~IloExcelReadOnlyError();
};

class IloExcelInstanceError : public IloException {
  IloInt _code;
public:
  IloExcelInstanceError();
public:
  virtual ~IloExcelInstanceError();
};
/**
* @group optim.concert.source
* @show longernal
*
* <p>Excel longernal.</p>
*/
class IloExcelConnection {
  enum {IloExcelErrSIZE = 1024};
  _ApplicationPtr _pXL;
  _WorkbookPtr _pBook;
  RangePtr _pRange;
  WorksheetFunctionPtr _pWsFunc;
  WorkbooksPtr _pWBooks;
  XlCalculation _calculation;
  IloBool _hasFormula;
private:
  IloExcelConnection(const IloExcelConnection&);
  void operator=(const IloExcelConnection&);
  void end();
public:
	IloBool hasFormula() const { return _hasFormula; }
//  IloExcelConnection(const char* xlsFile, const char* passwd, IloBool readOnly = IloFalse, IloBool visible = IloFalse);
  IloExcelConnection(const char* xlsFile, IloBool readOnly);
  ~IloExcelConnection();
  void setCurrentRange(const char* range);
  void setReference(long vertOffset, long horizOffset);
  IloInt getNumberOfRows();
  IloInt getNumberOfCols();

  IloIntArray readIntArray(IloEnvI* env, long col, long line);
  IloNumArray readNumArray(IloEnvI* env, long col, long line);
  IloAnyArray readSymbolArray(IloEnvI* env, long col, long line);
  void fillTupleSet(IloTupleCollectionI* set, long i, long j);

	void write(IloIntArray ar, long i, long j);
	void write(IloNumArray ar, long i, long j);
	void write(IloAnyArray ar, long i, long j);
	void write(IloTupleCollectionI* ar, long i, long j);
  IloBool isNumber(long vertOffset, long horizOffset);
  IloBool isString(long vertOffset, long horizOffset);
  IloBool isError(long vertOffset, long horizOffset);
  IloBool isError(_variant_t& var) const;
  /*BSTR for leftValue is allocated inside;
  it's the caller's job to free it :
  const bstring& getStringCell(IloInt vertOffset, IloInt horizOffset, bstring& leftValue);*/
  const char* getStringCell(long vertOffset, long horizOffset);//, char* leftValue);
  IloNum getFloatCell(long vertOffset, long horizOffset);
  IloInt getIntCell(long vertOffset, long horizOffset);

  void setStringCell(long vertOffset, long horizOffset, const char* value);
  void setFloatCell(long vertOffset, long horizOffset, IloNum value);
  void setIntCell(long vertOffset, long horizOffset, IloInt value);
  void save();

private:
  char* convert(_bstr_t text) {
    char* str = 0;
    size_t length = strlen((const char*)text);
    ILOPROTECTNEW(str, new char[length+1]);
    strncpy(str, (const char*)text,length);
    str[length] = '\0';
    return str;
  }
};

#endif // ILO_WIN32
#endif // ILO_MSVC

#endif




